@@ -11,21 +11,22 @@
 #include <stdlib.h>
 #include <string.h>
 
-#include "bfcli/chain.h"
-#include "bfcli/helper.h"
-#include "bfcli/opts.h"
-#include "bfcli/print.h"
-#include "bfcli/ruleset.h"
-#include "core/chain.h"
-#include "core/helper.h"
-#include "core/hook.h"
-#include "core/list.h"
-#include "core/logger.h"
-#include "core/request.h"
-#include "core/response.h"
-#include "core/set.h"
-#include "libbpfilter/bpfilter.h"
-#include "version.h"
+#include <bpfilter/bpfilter.h>
+#include <bpfilter/chain.h>
+#include <bpfilter/helper.h>
+#include <bpfilter/hook.h>
+#include <bpfilter/list.h>
+#include <bpfilter/logger.h>
+#include <bpfilter/request.h>
+#include <bpfilter/response.h>
+#include <bpfilter/set.h>
+#include <bpfilter/version.h>
+
+#include "chain.h"
+#include "helper.h"
+#include "opts.h"
+#include "print.h"
+#include "ruleset.h"
 
 static void _bfc_print_version(FILE *stream, struct argp_state *state)
 {
