@@ -3,11 +3,12 @@
  * Copyright (c) 2023 Meta Platforms, Inc. and affiliates.
  */
 
-#include "bfcli/print.h"
+#include "print.h"
 
 #include <linux/icmp.h>
 #include <linux/icmpv6.h>
 #include <linux/if_ether.h>
+#include <linux/in.h>
 #include <linux/ip.h>
 #include <linux/ipv6.h>
 #include <linux/tcp.h>
@@ -21,16 +22,17 @@
 #include <sys/socket.h>
 #include <time.h>
 
-#include "core/chain.h"
-#include "core/counter.h"
-#include "core/helper.h"
-#include "core/hook.h"
-#include "core/list.h"
-#include "core/logger.h"
-#include "core/matcher.h"
-#include "core/rule.h"
-#include "core/runtime.h"
-#include "core/verdict.h"
+#include <bpfilter/chain.h>
+#include <bpfilter/counter.h>
+#include <bpfilter/helper.h>
+#include <bpfilter/hook.h>
+#include <bpfilter/list.h>
+#include <bpfilter/logger.h>
+#include <bpfilter/matcher.h>
+#include <bpfilter/rule.h>
+#include <bpfilter/runtime.h>
+#include <bpfilter/set.h>
+#include <bpfilter/verdict.h>
 
 struct bfc_chain_opts;
 
@@ -141,6 +143,43 @@ void bfc_chain_dump(struct bf_chain *chain, struct bf_hookopts *hookopts,
     (void)fprintf(stdout, "error %lu packets %lu bytes\n", counter->packets,
                   counter->bytes);
 
+    // Loop over named sets
+    bf_list_foreach (&chain->sets, set_node) {
+        struct bf_set *set = bf_list_node_get_data(set_node);
+
+        if (!set->name)
+            continue;
+
+        (void)fprintf(stdout, "    set %s (", set->name);
+        for (size_t i = 0; i < set->n_comps; ++i) {
+            (void)fprintf(stdout, "%s", bf_matcher_type_to_str(set->key[i]));
+
+            if (i != set->n_comps - 1)
+                (void)fprintf(stdout, ", ");
+        }
+        (void)fprintf(stdout, ") in {\n");
+
+        bf_list_foreach (&set->elems, elem_node) {
+            uint32_t payload_idx = 0;
+            void *payload = bf_list_node_get_data(elem_node);
+
+            (void)fprintf(stdout, "        ");
+            for (size_t i = 0; i < set->n_comps; ++i) {
+                const struct bf_matcher_meta *meta =
+                    bf_matcher_get_meta(set->key[i]);
+
+                meta->ops[BF_MATCHER_IN].print(payload + payload_idx);
+                payload_idx += meta->ops[BF_MATCHER_IN].ref_payload_size;
+
+                if (i != set->n_comps - 1)
+                    (void)fprintf(stdout, ", ");
+            }
+            (void)fprintf(stdout, "\n");
+        }
+
+        (void)fprintf(stdout, "    }\n");
+    }
+
     // Loop over rules
     bf_list_foreach (&chain->rules, rule_node) {
         struct bf_rule *rule = bf_list_node_get_data(rule_node);
@@ -148,18 +187,62 @@ void bfc_chain_dump(struct bf_chain *chain, struct bf_hookopts *hookopts,
         (void)fprintf(stdout, "    rule\n");
         bf_list_foreach (&rule->matchers, matcher_node) {
             struct bf_matcher *matcher = bf_list_node_get_data(matcher_node);
-            const struct bf_matcher_ops *ops =
-                bf_matcher_get_ops(matcher->type, matcher->op);
+            const struct bf_matcher_ops *ops = bf_matcher_get_ops(
+                bf_matcher_get_type(matcher), bf_matcher_get_op(matcher));
 
-            (void)fprintf(stdout, "        %s",
-                          bf_matcher_type_to_str(matcher->type));
-            (void)fprintf(stdout, " %s ", bf_matcher_op_to_str(matcher->op));
+            if (bf_matcher_get_type(matcher) == BF_MATCHER_SET) {
+                struct bf_set *set =
+                    bf_chain_get_set_for_matcher(chain, matcher);
 
-            if (ops) {
-                ops->printer_cb(matcher);
+                (void)fprintf(stdout, "        (");
+                for (size_t i = 0; i < set->n_comps; ++i) {
+                    (void)fprintf(stdout, "%s",
+                                  bf_matcher_type_to_str(set->key[i]));
+
+                    if (i != set->n_comps - 1)
+                        (void)fprintf(stdout, ", ");
+                }
+
+                if (set->name) {
+                    (void)fprintf(stdout, ") in %s", set->name);
+                } else {
+                    (void)fprintf(stdout, ") in {\n");
+
+                    bf_list_foreach (&set->elems, elem_node) {
+                        uint32_t payload_idx = 0;
+                        void *payload = bf_list_node_get_data(elem_node);
+
+                        (void)fprintf(stdout, "            ");
+                        for (size_t i = 0; i < set->n_comps; ++i) {
+                            const struct bf_matcher_meta *meta =
+                                bf_matcher_get_meta(set->key[i]);
+
+                            meta->ops[BF_MATCHER_IN].print(payload +
+                                                           payload_idx);
+                            payload_idx +=
+                                meta->ops[BF_MATCHER_IN].ref_payload_size;
+
+                            if (i != set->n_comps - 1)
+                                (void)fprintf(stdout, ", ");
+                        }
+                        (void)fprintf(stdout, "\n");
+                    }
+
+                    (void)fprintf(stdout, "        }");
+                }
             } else {
-                bf_dump_hex_local(matcher->payload,
-                                  matcher->len - sizeof(struct bf_matcher));
+                (void)fprintf(
+                    stdout, "        %s",
+                    bf_matcher_type_to_str(bf_matcher_get_type(matcher)));
+                (void)fprintf(stdout, " %s ",
+                              bf_matcher_op_to_str(bf_matcher_get_op(matcher)));
+
+                if (ops) {
+                    ops->print(bf_matcher_payload(matcher));
+                } else {
+                    bf_dump_hex_local(bf_matcher_payload(matcher),
+                                      bf_matcher_payload_len(matcher));
+                }
             }
 
             (void)fprintf(stdout, "\n");
@@ -253,13 +336,13 @@ static void _bf_chain_log_header(const struct bf_log *log)
     (void)strftime(time_str, sizeof(time_str), "%H:%M:%S",
                    localtime(&time.tv_sec));
 
-    (void)fprintf(stdout, "\n%s[%s.%06ld]%s Packet: %s%llu bytes%s\n",
-                  bf_logger_get_color(BF_COLOR_LIGHT_CYAN, BF_STYLE_NORMAL),
-                  time_str, time.tv_nsec / BF_TIME_US,
-                  bf_logger_get_color(BF_COLOR_RESET, BF_STYLE_RESET),
-                  bf_logger_get_color(BF_COLOR_DEFAULT, BF_STYLE_BOLD),
-                  log->pkt_size,
-                  bf_logger_get_color(BF_COLOR_RESET, BF_STYLE_RESET));
+    (void)fprintf(
+        stdout, "\n%s[%s.%06ld]%s Rule #%u matched %s%llu bytes%s\n",
+        bf_logger_get_color(BF_COLOR_LIGHT_CYAN, BF_STYLE_NORMAL), time_str,
+        time.tv_nsec / BF_TIME_US,
+        bf_logger_get_color(BF_COLOR_RESET, BF_STYLE_RESET), log->rule_id,
+        bf_logger_get_color(BF_COLOR_DEFAULT, BF_STYLE_BOLD), log->pkt_size,
+        bf_logger_get_color(BF_COLOR_RESET, BF_STYLE_RESET));
 }
 
 static void _bf_chain_log_l2(const struct bf_log *log)
@@ -307,9 +390,9 @@ static void _bf_chain_log_l3(const struct bf_log *log)
         return;
     }
 
-    switch (be16toh(log->l3_proto)) {
+    switch (log->l3_proto) {
     case ETH_P_IP:
-        iphdr = (struct iphdr *)log->l3hdr;
+        iphdr = (struct iphdr *)&log->l3hdr[0];
 
         inet_ntop(AF_INET, &iphdr->saddr, src_addr, sizeof(src_addr));
         inet_ntop(AF_INET, &iphdr->daddr, dst_addr, sizeof(dst_addr));
@@ -362,7 +445,7 @@ static void _bf_chain_log_l3(const struct bf_log *log)
 
     default:
         (void)fprintf(stdout, "  Internet  : <unknown protocol 0x%04x>\n",
-                      be16toh(log->l3_proto));
+                      log->l3_proto);
     }
 }
 
