@@ -3,7 +3,7 @@
  * Copyright (c) 2023 Meta Platforms, Inc. and affiliates.
  */
 
-#include "bpfilter/cgen/cgroup.h"
+#include "cgen/cgroup.h"
 
 #include <linux/bpf_common.h>
 #include <linux/if_ether.h>
@@ -12,17 +12,17 @@
 #include <stdint.h>
 #include <sys/socket.h>
 
-#include "bpfilter/cgen/cgen.h"
-#include "bpfilter/cgen/program.h"
-#include "bpfilter/cgen/stub.h"
-#include "bpfilter/cgen/swich.h"
-#include "core/btf.h"
-#include "core/flavor.h"
-#include "core/helper.h"
-#include "core/verdict.h"
-#include "linux/bpf.h"
+#include <bpfilter/btf.h>
+#include <bpfilter/flavor.h>
+#include <bpfilter/helper.h>
+#include <bpfilter/verdict.h>
 
-#include "external/filter.h"
+#include "cgen/cgen.h"
+#include "cgen/program.h"
+#include "cgen/stub.h"
+#include "cgen/swich.h"
+#include "filter.h"
+#include "linux/bpf.h"
 
 // Forward definition to avoid headers clusterfuck.
 uint16_t htons(uint16_t hostshort);
@@ -34,12 +34,9 @@ static int _bf_cgroup_gen_inline_prologue(struct bf_program *program)
 
     bf_assert(program);
 
-    // Calculate the packet size (+ETH_HLEN) and store it into the runtime context
-    EMIT(program, BPF_LDX_MEM(BPF_W, BPF_REG_2, BPF_REG_1,
-                              offsetof(struct __sk_buff, data)));
+    // Copy the packet size (+ETH_HLEN) into the runtime context
     EMIT(program, BPF_LDX_MEM(BPF_W, BPF_REG_3, BPF_REG_1,
-                              offsetof(struct __sk_buff, data_end)));
-    EMIT(program, BPF_ALU64_REG(BPF_SUB, BPF_REG_3, BPF_REG_2));
+                              offsetof(struct __sk_buff, len)));
     EMIT(program, BPF_ALU64_IMM(BPF_ADD, BPF_REG_3, ETH_HLEN));
     EMIT(program,
          BPF_STX_MEM(BPF_DW, BPF_REG_10, BPF_REG_3, BF_PROG_CTX_OFF(pkt_size)));
@@ -105,6 +102,28 @@ static int _bf_cgroup_gen_inline_epilogue(struct bf_program *program)
     return 0;
 }
 
+static int _bf_cgroup_gen_inline_set_mark(struct bf_program *program,
+                                          uint32_t mark)
+{
+    EMIT(program,
+         BPF_LDX_MEM(BPF_DW, BPF_REG_1, BPF_REG_10, BF_PROG_CTX_OFF(arg)));
+    EMIT(program, BPF_MOV64_IMM(BPF_REG_2, mark));
+    EMIT(program, BPF_STX_MEM(BPF_W, BPF_REG_1, BPF_REG_2,
+                              offsetof(struct __sk_buff, mark)));
+
+    return 0;
+}
+
+static int _bf_cgroup_gen_inline_get_mark(struct bf_program *program, int reg)
+{
+    EMIT(program,
+         BPF_LDX_MEM(BPF_DW, BPF_REG_1, BPF_REG_10, BF_PROG_CTX_OFF(arg)));
+    EMIT(program,
+         BPF_LDX_MEM(BPF_W, reg, BPF_REG_1, offsetof(struct __sk_buff, mark)));
+
+    return 0;
+}
+
 /**
  * Convert a standard verdict into a return value.
  *
@@ -128,5 +147,7 @@ static int _bf_cgroup_get_verdict(enum bf_verdict verdict)
 const struct bf_flavor_ops bf_flavor_ops_cgroup = {
     .gen_inline_prologue = _bf_cgroup_gen_inline_prologue,
     .gen_inline_epilogue = _bf_cgroup_gen_inline_epilogue,
+    .gen_inline_set_mark = _bf_cgroup_gen_inline_set_mark,
+    .gen_inline_get_mark = _bf_cgroup_gen_inline_get_mark,
     .get_verdict = _bf_cgroup_get_verdict,
 };
