@@ -7,8 +7,9 @@
 
 #include <stddef.h>
 
-#include "bpfilter/cgen/elfstub.h"
-#include "core/dump.h"
+#include <bpfilter/dump.h>
+
+#include "cgen/elfstub.h"
 
 /**
  * Field to fixup in a @c bpf_insn structure.
