@@ -3,7 +3,7 @@
  * Copyright (c) 2022 Meta Platforms, Inc. and affiliates.
  */
 
-#include "bpfilter/cgen/matcher/meta.h"
+#include "cgen/matcher/meta.h"
 
 #include <linux/bpf.h>
 #include <linux/bpf_common.h>
@@ -16,21 +16,23 @@
 #include <stddef.h>
 #include <stdint.h>
 
-#include "bpfilter/cgen/program.h"
-#include "bpfilter/cgen/swich.h"
-#include "core/logger.h"
-#include "core/matcher.h"
+#include <bpfilter/logger.h>
+#include <bpfilter/matcher.h>
 
-#include "external/filter.h"
+#include "cgen/program.h"
+#include "cgen/swich.h"
+#include "filter.h"
 
+/** @todo Add support for input and output interface filtering based on the
+ * program's hook. */
 static int _bf_matcher_generate_meta_iface(struct bf_program *program,
                                            const struct bf_matcher *matcher)
 {
     EMIT(program,
          BPF_LDX_MEM(BPF_H, BPF_REG_1, BPF_REG_10, BF_PROG_CTX_OFF(ifindex)));
     EMIT_FIXUP_JMP_NEXT_RULE(
-        program,
-        BPF_JMP_IMM(BPF_JNE, BPF_REG_1, *(uint32_t *)&matcher->payload, 0));
+        program, BPF_JMP_IMM(BPF_JNE, BPF_REG_1,
+                             *(uint32_t *)bf_matcher_payload(matcher), 0));
 
     return 0;
 }
@@ -39,8 +41,9 @@ static int _bf_matcher_generate_meta_l3_proto(struct bf_program *program,
                                               const struct bf_matcher *matcher)
 {
     EMIT_FIXUP_JMP_NEXT_RULE(
-        program, BPF_JMP_IMM(BPF_JNE, BPF_REG_7,
-                             htobe16(*(uint16_t *)&matcher->payload), 0));
+        program,
+        BPF_JMP_IMM(BPF_JNE, BPF_REG_7,
+                    htobe16(*(uint16_t *)bf_matcher_payload(matcher)), 0));
 
     return 0;
 }
@@ -49,8 +52,10 @@ static int _bf_matcher_generate_meta_l4_proto(struct bf_program *program,
                                               const struct bf_matcher *matcher)
 {
     EMIT_FIXUP_JMP_NEXT_RULE(
-        program, BPF_JMP_IMM(matcher->op == BF_MATCHER_EQ ? BPF_JNE : BPF_JEQ,
-                             BPF_REG_8, *(uint8_t *)&matcher->payload, 0));
+        program,
+        BPF_JMP_IMM(bf_matcher_get_op(matcher) == BF_MATCHER_EQ ? BPF_JNE :
+                                                                  BPF_JEQ,
+                    BPF_REG_8, *(uint8_t *)bf_matcher_payload(matcher), 0));
 
     return 0;
 }
@@ -59,12 +64,13 @@ static int
 _bf_matcher_generate_meta_probability(struct bf_program *program,
                                       const struct bf_matcher *matcher)
 {
-    uint8_t proba = *(uint8_t *)matcher->payload;
+    uint8_t proba = *(uint8_t *)bf_matcher_payload(matcher);
 
     EMIT(program, BPF_EMIT_CALL(BPF_FUNC_get_prandom_u32));
     EMIT_FIXUP_JMP_NEXT_RULE(
-        program, BPF_JMP_IMM(BPF_JGT, BPF_REG_0,
-                             (int)(UINT32_MAX * (proba / 100.0)), 0));
+        program,
+        BPF_JMP_IMM(BPF_JGT, BPF_REG_0,
+                    (uint32_t)((uint64_t)UINT32_MAX * (proba / 100.0)), 0));
 
     return 0;
 }
@@ -73,7 +79,7 @@ static int _bf_matcher_generate_meta_port(struct bf_program *program,
                                           const struct bf_matcher *matcher)
 {
     _clean_bf_swich_ struct bf_swich swich;
-    uint16_t *port = (uint16_t *)&matcher->payload;
+    uint16_t *port = (uint16_t *)bf_matcher_payload(matcher);
     int r;
 
     // Load L4 header address into r6
@@ -82,16 +88,18 @@ static int _bf_matcher_generate_meta_port(struct bf_program *program,
 
     // Get the packet's port into r1
     swich = bf_swich_get(program, BPF_REG_8);
-    EMIT_SWICH_OPTION(&swich, IPPROTO_TCP,
-                      BPF_LDX_MEM(BPF_H, BPF_REG_1, BPF_REG_6,
-                                  matcher->type == BF_MATCHER_META_SPORT ?
-                                      offsetof(struct tcphdr, source) :
-                                      offsetof(struct tcphdr, dest)));
-    EMIT_SWICH_OPTION(&swich, IPPROTO_UDP,
-                      BPF_LDX_MEM(BPF_H, BPF_REG_1, BPF_REG_6,
-                                  matcher->type == BF_MATCHER_META_SPORT ?
-                                      offsetof(struct udphdr, source) :
-                                      offsetof(struct udphdr, dest)));
+    EMIT_SWICH_OPTION(
+        &swich, IPPROTO_TCP,
+        BPF_LDX_MEM(BPF_H, BPF_REG_1, BPF_REG_6,
+                    bf_matcher_get_type(matcher) == BF_MATCHER_META_SPORT ?
+                        offsetof(struct tcphdr, source) :
+                        offsetof(struct tcphdr, dest)));
+    EMIT_SWICH_OPTION(
+        &swich, IPPROTO_UDP,
+        BPF_LDX_MEM(BPF_H, BPF_REG_1, BPF_REG_6,
+                    bf_matcher_get_type(matcher) == BF_MATCHER_META_SPORT ?
+                        offsetof(struct udphdr, source) :
+                        offsetof(struct udphdr, dest)));
     EMIT_SWICH_DEFAULT(&swich, BPF_MOV64_IMM(BPF_REG_1, 0));
 
     r = bf_swich_generate(&swich);
@@ -101,14 +109,14 @@ static int _bf_matcher_generate_meta_port(struct bf_program *program,
     // If r1 == 0: no TCP nor UDP header found, jump to the next rule
     EMIT_FIXUP_JMP_NEXT_RULE(program, BPF_JMP_IMM(BPF_JEQ, BPF_REG_1, 0, 0));
 
-    switch (matcher->op) {
+    switch (bf_matcher_get_op(matcher)) {
     case BF_MATCHER_EQ:
-        EMIT_FIXUP_JMP_NEXT_RULE(
-            program, BPF_JMP_IMM(BPF_JNE, BPF_REG_1, htobe16(*port), 0));
+        EMIT_FIXUP_JMP_NEXT_RULE(program,
+                                 BPF_JMP_IMM(BPF_JNE, BPF_REG_1, *port, 0));
         break;
     case BF_MATCHER_NE:
-        EMIT_FIXUP_JMP_NEXT_RULE(
-            program, BPF_JMP_IMM(BPF_JEQ, BPF_REG_1, htobe16(*port), 0));
+        EMIT_FIXUP_JMP_NEXT_RULE(program,
+                                 BPF_JMP_IMM(BPF_JEQ, BPF_REG_1, *port, 0));
         break;
     case BF_MATCHER_RANGE:
         /* Convert the big-endian value stored in the packet into a
@@ -123,7 +131,36 @@ static int _bf_matcher_generate_meta_port(struct bf_program *program,
         break;
     default:
         return bf_err_r(-EINVAL, "unknown matcher operator '%s' (%d)",
-                        bf_matcher_op_to_str(matcher->op), matcher->op);
+                        bf_matcher_op_to_str(bf_matcher_get_op(matcher)),
+                        bf_matcher_get_op(matcher));
+    }
+
+    return 0;
+}
+
+static int _bf_matcher_generate_meta_mark(struct bf_program *program,
+                                          const struct bf_matcher *matcher)
+{
+    uint32_t mark = *(uint32_t *)bf_matcher_payload(matcher);
+    int r;
+
+    r = program->runtime.ops->gen_inline_get_mark(program, BPF_REG_1);
+    if (r)
+        return bf_err_r(r, "failed to get inline mark");
+
+    switch (bf_matcher_get_op(matcher)) {
+    case BF_MATCHER_EQ:
+        EMIT_FIXUP_JMP_NEXT_RULE(program,
+                                 BPF_JMP_IMM(BPF_JNE, BPF_REG_1, mark, 0));
+        break;
+    case BF_MATCHER_NE:
+        EMIT_FIXUP_JMP_NEXT_RULE(program,
+                                 BPF_JMP_IMM(BPF_JEQ, BPF_REG_1, mark, 0));
+        break;
+    default:
+        return bf_err_r(-EINVAL, "unknown matcher operator '%s' (%d)",
+                        bf_matcher_op_to_str(bf_matcher_get_op(matcher)),
+                        bf_matcher_get_op(matcher));
     }
 
     return 0;
@@ -134,7 +171,7 @@ int bf_matcher_generate_meta(struct bf_program *program,
 {
     int r;
 
-    switch (matcher->type) {
+    switch (bf_matcher_get_type(matcher)) {
     case BF_MATCHER_META_IFACE:
         r = _bf_matcher_generate_meta_iface(program, matcher);
         break;
@@ -151,8 +188,12 @@ int bf_matcher_generate_meta(struct bf_program *program,
     case BF_MATCHER_META_DPORT:
         r = _bf_matcher_generate_meta_port(program, matcher);
         break;
+    case BF_MATCHER_META_MARK:
+        r = _bf_matcher_generate_meta_mark(program, matcher);
+        break;
     default:
-        return bf_err_r(-EINVAL, "unknown matcher type %d", matcher->type);
+        return bf_err_r(-EINVAL, "unknown matcher type %d",
+                        bf_matcher_get_type(matcher));
     };
 
     if (r)
