@@ -3,7 +3,7 @@
  * Copyright (c) 2023 Meta Platforms, Inc. and affiliates.
  */
 
-#include "bpfilter/cgen/nf.h"
+#include "cgen/nf.h"
 
 #include <linux/bpf.h>
 #include <linux/bpf_common.h>
@@ -15,17 +15,17 @@
 #include <stdint.h>
 #include <sys/socket.h>
 
-#include "bpfilter/cgen/jmp.h"
-#include "bpfilter/cgen/program.h"
-#include "bpfilter/cgen/stub.h"
-#include "bpfilter/cgen/swich.h"
-#include "core/btf.h"
-#include "core/flavor.h"
-#include "core/helper.h"
-#include "core/hook.h"
-#include "core/verdict.h"
+#include <bpfilter/btf.h>
+#include <bpfilter/flavor.h>
+#include <bpfilter/helper.h>
+#include <bpfilter/hook.h>
+#include <bpfilter/verdict.h>
 
-#include "external/filter.h"
+#include "cgen/jmp.h"
+#include "cgen/program.h"
+#include "cgen/stub.h"
+#include "cgen/swich.h"
+#include "filter.h"
 
 #define BF_NF_PRIO_EVEN 2
 #define BF_NF_PRIO_ODD 1
@@ -124,6 +124,22 @@ static int _bf_nf_gen_inline_epilogue(struct bf_program *program)
     return 0;
 }
 
+static int _bf_nf_gen_inline_get_mark(struct bf_program *program, int reg)
+{
+    int offset;
+
+    EMIT(program,
+         BPF_LDX_MEM(BPF_DW, BPF_REG_1, BPF_REG_10, BF_PROG_CTX_OFF(arg)));
+    if ((offset = bf_btf_get_field_off("bpf_nf_ctx", "skb")) < 0)
+        return offset;
+    EMIT(program, BPF_LDX_MEM(BPF_DW, BPF_REG_2, BPF_REG_1, offset));
+    if ((offset = bf_btf_get_field_off("sk_buff", "mark")) < 0)
+        return offset;
+    EMIT(program, BPF_LDX_MEM(BPF_W, reg, BPF_REG_2, offset));
+
+    return 0;
+}
+
 /**
  * Convert a standard verdict into a return value.
  *
@@ -147,5 +163,6 @@ static int _bf_nf_get_verdict(enum bf_verdict verdict)
 const struct bf_flavor_ops bf_flavor_ops_nf = {
     .gen_inline_prologue = _bf_nf_gen_inline_prologue,
     .gen_inline_epilogue = _bf_nf_gen_inline_epilogue,
+    .gen_inline_get_mark = _bf_nf_gen_inline_get_mark,
     .get_verdict = _bf_nf_get_verdict,
 };
