@@ -3,9 +3,10 @@
  * Copyright (c) 2022 Meta Platforms, Inc. and affiliates.
  */
 
-#include "bpfilter/cgen/prog/link.h"
+#include "cgen/prog/link.h"
 
 #include <linux/bpf.h>
+#include <linux/if_link.h>
 
 #include <errno.h>
 #include <fcntl.h>
@@ -14,13 +15,13 @@
 #include <string.h>
 #include <unistd.h>
 
-#include "core/bpf.h"
-#include "core/dump.h"
-#include "core/flavor.h"
-#include "core/helper.h"
-#include "core/hook.h"
-#include "core/logger.h"
-#include "core/marsh.h"
+#include <bpfilter/bpf.h>
+#include <bpfilter/dump.h>
+#include <bpfilter/flavor.h>
+#include <bpfilter/helper.h>
+#include <bpfilter/hook.h>
+#include <bpfilter/logger.h>
+#include <bpfilter/pack.h>
 
 int bf_link_new(struct bf_link **link, const char *name)
 {
@@ -43,31 +44,29 @@ int bf_link_new(struct bf_link **link, const char *name)
     return 0;
 }
 
-int bf_link_new_from_marsh(struct bf_link **link, int dir_fd,
-                           const struct bf_marsh *marsh)
+int bf_link_new_from_pack(struct bf_link **link, int dir_fd,
+                          bf_rpack_node_t node)
 {
     _free_bf_link_ struct bf_link *_link = NULL;
-    _free_bf_hookopts_ struct bf_hookopts *hookopts = NULL;
-    struct bf_marsh *child = NULL;
+    _cleanup_free_ char *name = NULL;
+    bf_rpack_node_t child;
     int r;
 
-    bf_assert(link && marsh);
+    bf_assert(link);
 
-    _link = malloc(sizeof(*_link));
-    if (!_link)
-        return -ENOMEM;
+    r = bf_rpack_kv_str(node, "name", &name);
+    if (r)
+        return bf_rpack_key_err(r, "bf_link.name");
 
-    _link->hookopts = NULL;
-    _link->fd = -1;
+    r = bf_link_new(&_link, name);
+    if (r)
+        return bf_err_r(r, "failed to create bf_link from pack");
 
-    if (!(child = bf_marsh_next_child(marsh, child)))
-        return -EINVAL;
-    memcpy(&_link->name, child->data, BPF_OBJ_NAME_LEN);
-
-    if (!(child = bf_marsh_next_child(marsh, child)))
-        return -EINVAL;
-    if (!bf_marsh_is_empty(child)) {
-        r = bf_hookopts_new_from_marsh(&_link->hookopts, child);
+    r = bf_rpack_kv_node(node, "hookopts", &child);
+    if (r)
+        return bf_rpack_key_err(r, "bf_link.hookopts");
+    if (!bf_rpack_is_nil(child)) {
+        r = bf_hookopts_new_from_pack(&_link->hookopts, child);
         if (r)
             return r;
     }
@@ -97,41 +96,22 @@ void bf_link_free(struct bf_link **link)
     freep((void *)link);
 }
 
-int bf_link_marsh(const struct bf_link *link, struct bf_marsh **marsh)
+int bf_link_pack(const struct bf_link *link, bf_wpack_t *pack)
 {
-    _free_bf_marsh_ struct bf_marsh *_marsh = NULL;
-    int r;
+    bf_assert(link);
+    bf_assert(pack);
 
-    bf_assert(link && marsh);
+    bf_wpack_kv_str(pack, "name", link->name);
 
-    r = bf_marsh_new(&_marsh, NULL, 0);
-    if (r)
-        return r;
-
-    r = bf_marsh_add_child_raw(&_marsh, &link->name, BPF_OBJ_NAME_LEN);
-    if (r)
-        return r;
-
-    // Serialize link.hookopts
     if (link->hookopts) {
-        _free_bf_marsh_ struct bf_marsh *hookopts_elem = NULL;
-
-        r = bf_hookopts_marsh(link->hookopts, &hookopts_elem);
-        if (r < 0)
-            return r;
-
-        r = bf_marsh_add_child_obj(&_marsh, hookopts_elem);
-        if (r < 0)
-            return r;
+        bf_wpack_open_object(pack, "hookopts");
+        bf_hookopts_pack(link->hookopts, pack);
+        bf_wpack_close_object(pack);
     } else {
-        r = bf_marsh_add_child_raw(&_marsh, NULL, 0);
-        if (r)
-            return r;
+        bf_wpack_kv_nil(pack, "hookopts");
     }
 
-    *marsh = TAKE_PTR(_marsh);
-
-    return 0;
+    return bf_wpack_is_valid(pack) ? 0 : -EINVAL;
 }
 
 void bf_link_dump(const struct bf_link *link, prefix_t *prefix)
@@ -156,194 +136,84 @@ void bf_link_dump(const struct bf_link *link, prefix_t *prefix)
     bf_dump_prefix_pop(prefix);
 }
 
-static int _bf_link_attach_xdp(struct bf_link *link, enum bf_hook hook,
-                               const struct bf_hookopts *hookopts, int prog_fd)
-{
-    union bpf_attr attr;
-    int r;
-
-    UNUSED(hook);
-
-    memset(&attr, 0, sizeof(attr));
-
-    attr.link_create.prog_fd = prog_fd;
-    attr.link_create.target_fd = hookopts->ifindex;
-    attr.link_create.attach_type = BPF_XDP;
-    attr.link_create.flags = BF_XDP_MODE_SKB;
-
-    r = bf_bpf(BPF_LINK_CREATE, &attr);
-    if (r < 0)
-        return r;
-
-    link->fd = r;
-
-    return 0;
-}
-
-static int _bf_link_attach_tc(struct bf_link *link, enum bf_hook hook,
-                              const struct bf_hookopts *hookopts, int prog_fd)
-{
-    union bpf_attr attr;
-    int r;
-
-    UNUSED(hook);
-
-    memset(&attr, 0, sizeof(attr));
-
-    attr.link_create.prog_fd = prog_fd;
-    attr.link_create.target_fd = hookopts->ifindex;
-    attr.link_create.attach_type = bf_hook_to_bpf_attach_type(hook);
-
-    r = bf_bpf(BPF_LINK_CREATE, &attr);
-    if (r < 0)
-        return r;
-
-    link->fd = r;
-
-    return 0;
-}
-
-static int _bf_link_attach_nf(struct bf_link *link, enum bf_hook hook,
-                              const struct bf_hookopts *hookopts, int prog_fd)
-{
-    union bpf_attr attr;
-    int r;
-
-    memset(&attr, 0, sizeof(attr));
-
-    attr.link_create.prog_fd = prog_fd;
-    attr.link_create.attach_type = BPF_NETFILTER;
-    attr.link_create.netfilter.pf = hookopts->family;
-    attr.link_create.netfilter.hooknum = bf_hook_to_nf_hook(hook);
-    attr.link_create.netfilter.priority = hookopts->priorities[0];
-
-    r = bf_bpf(BPF_LINK_CREATE, &attr);
-    if (r < 0)
-        return r;
-
-    link->fd = r;
-
-    return 0;
-}
-
-static int _bf_link_attach_cgroup(struct bf_link *link, enum bf_hook hook,
-                                  const struct bf_hookopts *hookopts,
-                                  int prog_fd)
-{
-    _cleanup_close_ int cgroup_fd = -1;
-    union bpf_attr attr;
-    int r;
-
-    memset(&attr, 0, sizeof(attr));
-
-    attr.link_create.prog_fd = prog_fd;
-    attr.link_create.attach_type = bf_hook_to_bpf_attach_type(hook);
-
-    cgroup_fd = open(hookopts->cgpath, O_DIRECTORY | O_RDONLY);
-    if (cgroup_fd < 0)
-        return bf_err_r(errno, "failed to open cgroup '%s'", hookopts->cgpath);
-
-    attr.link_create.target_fd = cgroup_fd;
-
-    r = bf_bpf(BPF_LINK_CREATE, &attr);
-    if (r < 0)
-        return r;
-
-    link->fd = r;
-
-    return 0;
-}
-
 int bf_link_attach(struct bf_link *link, enum bf_hook hook,
                    struct bf_hookopts **hookopts, int prog_fd)
 {
-    int r;
-
     bf_assert(link && hookopts);
 
+    _cleanup_close_ int cgroup_fd = -1;
+    struct bf_hookopts *_hookopts = *hookopts;
+    int r;
+
     switch (bf_hook_to_flavor(hook)) {
     case BF_FLAVOR_XDP:
-        r = _bf_link_attach_xdp(link, hook, *hookopts, prog_fd);
+        r = bf_bpf_link_create(prog_fd, _hookopts->ifindex, hook, _hookopts,
+                               XDP_FLAGS_SKB_MODE);
         break;
     case BF_FLAVOR_TC:
-        r = _bf_link_attach_tc(link, hook, *hookopts, prog_fd);
+        r = bf_bpf_link_create(prog_fd, _hookopts->ifindex, hook, _hookopts, 0);
         break;
     case BF_FLAVOR_CGROUP:
-        r = _bf_link_attach_cgroup(link, hook, *hookopts, prog_fd);
+        cgroup_fd = open(_hookopts->cgpath, O_DIRECTORY | O_RDONLY);
+        if (cgroup_fd < 0) {
+            return bf_err_r(errno, "failed to open cgroup '%s'",
+                            _hookopts->cgpath);
+        }
+
+        r = bf_bpf_link_create(prog_fd, cgroup_fd, hook, _hookopts, 0);
         break;
     case BF_FLAVOR_NF:
-        r = _bf_link_attach_nf(link, hook, *hookopts, prog_fd);
+        r = bf_bpf_link_create(prog_fd, 0, hook, _hookopts, 0);
         break;
     default:
         return -ENOTSUP;
     }
 
-    if (r)
+    if (r < 0)
         return r;
 
+    link->fd = r;
     link->hookopts = TAKE_PTR(*hookopts);
 
     return 0;
 }
 
-static int _bf_link_update(struct bf_link *link, enum bf_hook hook, int prog_fd)
-{
-    union bpf_attr attr;
-
-    UNUSED(hook);
-
-    bf_assert(link);
-
-    memset(&attr, 0, sizeof(attr));
-
-    attr.link_update.link_fd = link->fd;
-    attr.link_update.new_prog_fd = prog_fd;
-
-    return bf_bpf(BPF_LINK_UPDATE, &attr);
-}
-
 static int _bf_link_update_nf(struct bf_link *link, enum bf_hook hook,
                               int prog_fd)
 {
-    _cleanup_close_ int new_link_fd = -1;
-    union bpf_attr attr;
-    int priorities[2];
-
     bf_assert(link);
 
-    memset(&attr, 0, sizeof(attr));
+    _cleanup_close_ int new_link_fd = -1;
+    struct bf_hookopts opts = *link->hookopts;
 
-    attr.link_create.prog_fd = prog_fd;
-    attr.link_create.attach_type = BPF_NETFILTER;
+    opts.priorities[0] = link->hookopts->priorities[1];
+    opts.priorities[1] = link->hookopts->priorities[0];
 
-    memcpy(priorities, link->hookopts->priorities, sizeof(priorities));
-
-    attr.link_create.netfilter.pf = link->hookopts->family;
-    attr.link_create.netfilter.hooknum = bf_hook_to_nf_hook(hook);
-    attr.link_create.netfilter.priority = priorities[1];
-
-    new_link_fd = bf_bpf(BPF_LINK_CREATE, &attr);
+    new_link_fd = bf_bpf_link_create(prog_fd, 0, hook, &opts, 0);
     if (new_link_fd < 0)
         return new_link_fd;
 
     // Swap priorities, so priorities[0] is the one currently used
-    link->hookopts->priorities[0] = priorities[1];
-    link->hookopts->priorities[1] = priorities[0];
+    link->hookopts->priorities[0] = opts.priorities[0];
+    link->hookopts->priorities[1] = opts.priorities[1];
+
+    closep(&link->fd);
+    link->fd = TAKE_FD(new_link_fd);
 
     return 0;
 }
 
 int bf_link_update(struct bf_link *link, enum bf_hook hook, int prog_fd)
 {
-    int r;
-
     bf_assert(link);
 
+    int r;
+
     switch (bf_hook_to_flavor(hook)) {
     case BF_FLAVOR_XDP:
     case BF_FLAVOR_TC:
     case BF_FLAVOR_CGROUP:
-        r = _bf_link_update(link, hook, prog_fd);
+        r = bf_bpf_link_update(link->fd, prog_fd);
         break;
     case BF_FLAVOR_NF:
         r = _bf_link_update_nf(link, hook, prog_fd);
@@ -357,16 +227,11 @@ int bf_link_update(struct bf_link *link, enum bf_hook hook, int prog_fd)
 
 void bf_link_detach(struct bf_link *link)
 {
-    union bpf_attr attr;
-    int r;
-
     bf_assert(link);
 
-    memset(&attr, 0, sizeof(attr));
+    int r;
 
-    attr.link_detach.link_fd = link->fd;
-
-    r = bf_bpf(BPF_LINK_DETACH, &attr);
+    r = bf_bpf_link_detach(link->fd);
     if (r) {
         bf_warn_r(
             r,
