@@ -3,7 +3,7 @@
  * Copyright (c) 2022 Meta Platforms, Inc. and affiliates.
  */
 
-#include "bpfilter/cgen/prog/map.h"
+#include "cgen/prog/map.h"
 
 #include <linux/bpf.h>
 
@@ -15,33 +15,39 @@
 #include <string.h>
 #include <unistd.h>
 
-#include "bpfilter/ctx.h"
-#include "core/bpf.h"
-#include "core/dump.h"
-#include "core/helper.h"
-#include "core/logger.h"
-#include "core/marsh.h"
+#include <bpfilter/bpf.h>
+#include <bpfilter/bpf_types.h>
+#include <bpfilter/btf.h>
+#include <bpfilter/dump.h>
+#include <bpfilter/helper.h>
+#include <bpfilter/logger.h>
 
-int bf_map_new(struct bf_map **map, const char *name, enum bf_map_type type,
-               enum bf_map_bpf_type bpf_type, size_t key_size,
-               size_t value_size, size_t n_elems)
+#include "ctx.h"
+
+static int _bf_map_new(struct bf_map **map, const char *name,
+                       enum bf_map_type type, enum bf_bpf_map_type bpf_type,
+                       size_t key_size, size_t value_size, size_t n_elems)
 {
+    bf_assert(map && name);
+
     _free_bf_map_ struct bf_map *_map = NULL;
 
-    bf_assert(map && name);
-    bf_assert(name[0] != '\0');
-    bf_assert(n_elems > 0);
+    if (name[0] == '\0')
+        return bf_err_r(-EINVAL, "map name can't be empty");
+
+    if (n_elems == 0)
+        return bf_err_r(-EINVAL, "map can't be created without elements");
 
     _map = malloc(sizeof(*_map));
     if (!_map)
         return -ENOMEM;
 
     _map->type = type;
-    _map->fd = -1;
     _map->bpf_type = bpf_type;
     _map->key_size = key_size;
     _map->value_size = value_size;
     _map->n_elems = n_elems;
+    _map->fd = -1;
 
     bf_strncpy(_map->name, BPF_OBJ_NAME_LEN, name);
 
@@ -50,52 +56,84 @@ int bf_map_new(struct bf_map **map, const char *name, enum bf_map_type type,
     return 0;
 }
 
-int bf_map_new_from_marsh(struct bf_map **map, int dir_fd,
-                          const struct bf_marsh *marsh)
+int bf_map_new(struct bf_map **map, const char *name, enum bf_map_type type,
+               size_t key_size, size_t value_size, size_t n_elems)
+{
+    static enum bf_bpf_map_type _map_type_to_bpf[_BF_MAP_TYPE_MAX] = {
+        [BF_MAP_TYPE_COUNTERS] = BF_BPF_MAP_TYPE_ARRAY,
+        [BF_MAP_TYPE_PRINTER] = BF_BPF_MAP_TYPE_ARRAY,
+        [BF_MAP_TYPE_LOG] = BF_BPF_MAP_TYPE_RINGBUF,
+    };
+
+    if (type == BF_MAP_TYPE_SET) {
+        return bf_err_r(
+            -EINVAL,
+            "use bf_map_new_from_set() to create a bf_map from a bf_set");
+    }
+
+    return _bf_map_new(map, name, type, _map_type_to_bpf[type], key_size,
+                       value_size, n_elems);
+}
+
+int bf_map_new_from_set(struct bf_map **map, const char *name,
+                        const struct bf_set *set)
+{
+    return _bf_map_new(map, name, BF_MAP_TYPE_SET,
+                       set->use_trie ? BF_BPF_MAP_TYPE_LPM_TRIE :
+                                       BF_BPF_MAP_TYPE_HASH,
+                       set->elem_size, 1, bf_list_size(&set->elems));
+}
+
+int bf_map_new_from_pack(struct bf_map **map, int dir_fd, bf_rpack_node_t node)
 {
     _free_bf_map_ struct bf_map *_map = NULL;
-    struct bf_marsh *elem = NULL;
+    _cleanup_free_ char *name = NULL;
     int r;
 
     bf_assert(map);
-    bf_assert(marsh);
 
     _map = malloc(sizeof(*_map));
     if (!_map)
         return -ENOMEM;
 
-    _map->fd = -1;
+    r = bf_rpack_kv_str(node, "name", &name);
+    if (r)
+        return bf_rpack_key_err(r, "bf_map.name");
+    if (strlen(name) == 0)
+        return bf_err_r(-EINVAL, "map name can't be empty");
+    bf_strncpy(_map->name, BPF_OBJ_NAME_LEN, name);
 
-    if (!(elem = bf_marsh_next_child(marsh, elem)))
-        return -EINVAL;
-    memcpy(&_map->type, elem->data, sizeof(_map->type));
+    r = bf_rpack_kv_enum(node, "type", &_map->type);
+    if (r)
+        return bf_rpack_key_err(r, "bf_map.type");
 
-    if (!(elem = bf_marsh_next_child(marsh, elem)))
-        return -EINVAL;
-    memcpy(_map->name, elem->data, BPF_OBJ_NAME_LEN);
+    r = bf_rpack_kv_enum(node, "bpf_type", &_map->bpf_type);
+    if (r)
+        return bf_rpack_key_err(r, "bf_map.bpf_type");
 
-    if (!(elem = bf_marsh_next_child(marsh, elem)))
-        return -EINVAL;
-    memcpy(&_map->bpf_type, elem->data, sizeof(_map->bpf_type));
+    r = bf_rpack_kv_u64(node, "key_size", &_map->key_size);
+    if (r)
+        return bf_rpack_key_err(r, "bf_map.key_size");
 
-    if (!(elem = bf_marsh_next_child(marsh, elem)))
-        return -EINVAL;
-    memcpy(&_map->key_size, elem->data, sizeof(_map->key_size));
+    r = bf_rpack_kv_u64(node, "value_size", &_map->value_size);
+    if (r)
+        return bf_rpack_key_err(r, "bf_map.value_size");
 
-    if (!(elem = bf_marsh_next_child(marsh, elem)))
-        return -EINVAL;
-    memcpy(&_map->value_size, elem->data, sizeof(_map->value_size));
+    r = bf_rpack_kv_u64(node, "n_elems", &_map->n_elems);
+    if (r)
+        return bf_rpack_key_err(r, "bf_map.n_elems");
+    if (_map->n_elems == 0)
+        return bf_err_r(-EINVAL, "bf_map should not have 0 elements");
 
-    if (!(elem = bf_marsh_next_child(marsh, elem)))
-        return -EINVAL;
-    memcpy(&_map->n_elems, elem->data, sizeof(_map->n_elems));
-
-    if (bf_marsh_next_child(marsh, elem))
-        return bf_err_r(-E2BIG, "too many elements in bf_map marsh");
-
-    r = bf_bpf_obj_get(_map->name, dir_fd, &_map->fd);
-    if (r < 0)
-        return bf_err_r(r, "failed to open pinned BPF map '%s'", _map->name);
+    if (dir_fd != -1) {
+        r = bf_bpf_obj_get(_map->name, dir_fd, &_map->fd);
+        if (r < 0) {
+            return bf_err_r(r, "failed to open pinned BPF map '%s'",
+                            _map->name);
+        }
+    } else {
+        _map->fd = -1;
+    }
 
     *map = TAKE_PTR(_map);
 
@@ -113,46 +151,19 @@ void bf_map_free(struct bf_map **map)
     freep((void *)map);
 }
 
-int bf_map_marsh(const struct bf_map *map, struct bf_marsh **marsh)
+int bf_map_pack(const struct bf_map *map, bf_wpack_t *pack)
 {
-    _free_bf_marsh_ struct bf_marsh *_marsh = NULL;
-    int r;
-
     bf_assert(map);
-    bf_assert(marsh);
+    bf_assert(pack);
 
-    r = bf_marsh_new(&_marsh, NULL, 0);
-    if (r < 0)
-        return r;
+    bf_wpack_kv_str(pack, "name", map->name);
+    bf_wpack_kv_enum(pack, "type", map->type);
+    bf_wpack_kv_enum(pack, "bpf_type", map->bpf_type);
+    bf_wpack_kv_u64(pack, "key_size", map->key_size);
+    bf_wpack_kv_u64(pack, "value_size", map->value_size);
+    bf_wpack_kv_u64(pack, "n_elems", map->n_elems);
 
-    r = bf_marsh_add_child_raw(&_marsh, &map->type, sizeof(map->type));
-    if (r < 0)
-        return r;
-
-    r = bf_marsh_add_child_raw(&_marsh, map->name, BPF_OBJ_NAME_LEN);
-    if (r < 0)
-        return r;
-
-    r = bf_marsh_add_child_raw(&_marsh, &map->bpf_type, sizeof(map->bpf_type));
-    if (r < 0)
-        return r;
-
-    r = bf_marsh_add_child_raw(&_marsh, &map->key_size, sizeof(map->key_size));
-    if (r < 0)
-        return r;
-
-    r = bf_marsh_add_child_raw(&_marsh, &map->value_size,
-                               sizeof(map->value_size));
-    if (r < 0)
-        return r;
-
-    r = bf_marsh_add_child_raw(&_marsh, &map->n_elems, sizeof(map->n_elems));
-    if (r < 0)
-        return r;
-
-    *marsh = TAKE_PTR(_marsh);
-
-    return 0;
+    return bf_wpack_is_valid(pack) ? 0 : -EINVAL;
 }
 
 static const char *_bf_map_type_to_str(enum bf_map_type type)
@@ -160,6 +171,7 @@ static const char *_bf_map_type_to_str(enum bf_map_type type)
     static const char *type_strs[] = {
         [BF_MAP_TYPE_COUNTERS] = "BF_MAP_TYPE_COUNTERS",
         [BF_MAP_TYPE_PRINTER] = "BF_MAP_TYPE_PRINTER",
+        [BF_MAP_TYPE_LOG] = "BF_MAP_TYPE_LOG",
         [BF_MAP_TYPE_SET] = "BF_MAP_TYPE_SET",
     };
 
@@ -170,6 +182,18 @@ static const char *_bf_map_type_to_str(enum bf_map_type type)
     return type_strs[type];
 }
 
+static const char *_bf_bpf_type_to_str(enum bf_bpf_map_type type)
+{
+    static const char *type_strs[] = {
+        [BF_BPF_MAP_TYPE_HASH] = "BF_BPF_MAP_TYPE_HASH",
+        [BF_BPF_MAP_TYPE_ARRAY] = "BF_BPF_MAP_TYPE_ARRAY",
+        [BF_BPF_MAP_TYPE_LPM_TRIE] = "BF_BPF_MAP_TYPE_LPM_TRIE",
+        [BF_BPF_MAP_TYPE_RINGBUF] = "BF_BPF_MAP_TYPE_RINGBUF",
+    };
+
+    return type_strs[type] ? type_strs[type] : "<no mapping>";
+}
+
 void bf_map_dump(const struct bf_map *map, prefix_t *prefix)
 {
     bf_assert(map);
@@ -179,47 +203,25 @@ void bf_map_dump(const struct bf_map *map, prefix_t *prefix)
 
     bf_dump_prefix_push(prefix);
     DUMP(prefix, "type: %s", _bf_map_type_to_str(map->type));
-    DUMP(prefix, "fd: %d", map->fd);
     DUMP(prefix, "name: %s", map->name);
-    DUMP(prefix, "bpf_type: %s", bf_map_bpf_type_to_str(map->bpf_type));
+    DUMP(prefix, "bpf_type: %s", _bf_bpf_type_to_str(map->bpf_type));
     DUMP(prefix, "key_size: %lu", map->key_size);
     DUMP(prefix, "value_size: %lu", map->value_size);
 
-    bf_dump_prefix_last(prefix);
     if (map->n_elems == BF_MAP_N_ELEMS_UNKNOWN) {
         DUMP(prefix, "n_elems: unknown");
     } else {
         DUMP(prefix, "n_elems: %lu", map->n_elems);
     }
 
+    bf_dump_prefix_last(prefix);
+    DUMP(prefix, "fd: %d", map->fd);
+
     bf_dump_prefix_pop(prefix);
 }
 
-static enum bpf_map_type
-_bf_map_bpf_type_to_kernel_type(enum bf_map_bpf_type bpf_type)
-{
-    static const enum bpf_map_type _kernel_types[] = {
-        [BF_MAP_BPF_TYPE_ARRAY] = BPF_MAP_TYPE_ARRAY,
-        [BF_MAP_BPF_TYPE_HASH] = BPF_MAP_TYPE_HASH,
-        [BF_MAP_BPF_TYPE_LPM_TRIE] = BPF_MAP_TYPE_LPM_TRIE,
-        [BF_MAP_BPF_TYPE_RINGBUF] = BPF_MAP_TYPE_RINGBUF,
-    };
-
-    bf_assert(0 <= bpf_type && bpf_type < _BF_MAP_BPF_TYPE_MAX);
-
-    return _kernel_types[bpf_type];
-}
-
 #define _free_bf_btf_ __attribute__((__cleanup__(_bf_btf_free)))
 
-struct bf_btf
-{
-    struct btf *btf;
-    uint32_t key_type_id;
-    uint32_t value_type_id;
-    int fd;
-};
-
 static void _bf_btf_free(struct bf_btf **btf);
 
 static int _bf_btf_new(struct bf_btf **btf)
@@ -258,7 +260,6 @@ static void _bf_btf_free(struct bf_btf **btf)
 static int _bf_btf_load(struct bf_btf *btf)
 {
     union bpf_attr attr = {};
-    int token_fd;
     const void *raw;
     int r;
 
@@ -268,17 +269,9 @@ static int _bf_btf_load(struct bf_btf *btf)
     if (!raw)
         return bf_err_r(errno, "failed to request BTF raw data");
 
-    attr.btf = bf_ptr_to_u64(raw);
-
-    token_fd = bf_ctx_token();
-    if (token_fd != -1) {
-        attr.btf_token_fd = token_fd;
-        attr.btf_flags |= BPF_F_TOKEN_FD;
-    }
-
-    r = bf_bpf(BPF_BTF_LOAD, &attr);
+    r = bf_bpf_btf_load(raw, bf_ctx_token());
     if (r < 0)
-        return bf_err_r(r, "failed to load BTF data");
+        return r;
 
     btf->fd = r;
 
@@ -327,17 +320,15 @@ static struct bf_btf *_bf_map_make_btf(const struct bf_map *map)
 
     r = _bf_btf_load(btf);
     if (r) {
-        bf_err_r(r, "failed to load BTF data");
+        bf_warn_r(r, "failed to load BTF data for %s, ignoring", map->name);
         return NULL;
     }
 
     return TAKE_PTR(btf);
 }
 
-int bf_map_create(struct bf_map *map, uint32_t flags)
+int bf_map_create(struct bf_map *map)
 {
-    int token_fd;
-    union bpf_attr attr = {};
     _free_bf_btf_ struct bf_btf *btf = NULL;
     int r;
 
@@ -361,21 +352,6 @@ int bf_map_create(struct bf_map *map, uint32_t flags)
             "can't create a map with BF_MAP_N_ELEMS_UNKNOWN number of elements");
     }
 
-    attr.map_type = _bf_map_bpf_type_to_kernel_type(map->bpf_type);
-    attr.key_size = map->key_size;
-    attr.value_size = map->value_size;
-    attr.max_entries = map->n_elems;
-    attr.map_flags = flags;
-
-    // NO_PREALLOC is *required* for LPM_TRIE map
-    if (map->bpf_type == BF_MAP_BPF_TYPE_LPM_TRIE)
-        attr.map_flags |= BPF_F_NO_PREALLOC;
-
-    if ((token_fd = bf_ctx_token()) != -1) {
-        attr.map_token_fd = token_fd;
-        attr.map_flags |= BPF_F_TOKEN_FD;
-    }
-
     /** The BTF data is not mandatory to use the map, but a good addition.
      * Hence, bpfilter will try to make the BTF data available, but will
      * ignore if that fails. @ref _bf_map_make_btf is used to isolate the
@@ -384,16 +360,9 @@ int bf_map_create(struct bf_map *map, uint32_t flags)
      * There is some boilerplate for @ref bf_btf structure, it could be
      * simpler, but the current implementation ensure the BTF data is properly
      * freed on error, without preventing the BPF map to be created. */
-    btf = _bf_map_make_btf(map);
-    if (btf) {
-        attr.btf_fd = btf->fd;
-        attr.btf_key_type_id = btf->key_type_id;
-        attr.btf_value_type_id = btf->value_type_id;
-    }
-
-    (void)snprintf(attr.map_name, BPF_OBJ_NAME_LEN, "%s", map->name);
-
-    r = bf_bpf(BPF_MAP_CREATE, &attr);
+    r = bf_bpf_map_create(map->name, map->bpf_type, map->key_size,
+                          map->value_size, map->n_elems, _bf_map_make_btf(map),
+                          bf_ctx_token());
     if (r < 0)
         return bf_err_r(r, "failed to create BPF map '%s'", map->name);
 
@@ -485,61 +454,7 @@ int bf_map_set_n_elems(struct bf_map *map, size_t n_elems)
 
 int bf_map_set_elem(const struct bf_map *map, void *key, void *value)
 {
-    union bpf_attr attr = {};
-
     bf_assert(map && key && value);
 
-    attr.map_fd = map->fd;
-    attr.key = (unsigned long long)key;
-    attr.value = (unsigned long long)value;
-    attr.flags = BPF_ANY;
-
-    return bf_bpf(BPF_MAP_UPDATE_ELEM, &attr);
-}
-
-static const char *_bf_map_bpf_type_strs[] = {
-    [BF_MAP_BPF_TYPE_ARRAY] = "BF_MAP_BPF_TYPE_ARRAY",
-    [BF_MAP_BPF_TYPE_HASH] = "BF_MAP_BPF_TYPE_HASH",
-    [BF_MAP_BPF_TYPE_LPM_TRIE] = "BF_MAP_BPF_TYPE_LPM_TRIE",
-    [BF_MAP_BPF_TYPE_RINGBUF] = "BF_MAP_BPF_TYPE_RINGBUF",
-};
-
-static_assert(ARRAY_SIZE(_bf_map_bpf_type_strs) == _BF_MAP_BPF_TYPE_MAX,
-              "missing entries in _bf_map_bpf_type_strs array");
-
-const char *bf_map_bpf_type_to_str(enum bf_map_bpf_type bpf_type)
-{
-    bf_assert(0 <= bpf_type && bpf_type < _BF_MAP_BPF_TYPE_MAX);
-
-    return _bf_map_bpf_type_strs[bpf_type];
-}
-
-int bf_map_bpf_type_from_str(const char *str, enum bf_map_bpf_type *bpf_type)
-{
-    bf_assert(str);
-    bf_assert(bpf_type);
-
-    for (size_t i = 0; i < _BF_MAP_BPF_TYPE_MAX; ++i) {
-        if (bf_streq(_bf_map_bpf_type_strs[i], str)) {
-            *bpf_type = i;
-            return 0;
-        }
-    }
-
-    return -EINVAL;
-}
-
-static enum bf_map_bpf_type _bf_map_bpf_type_to_set_type[] = {
-    [BF_SET_IP4] = BF_MAP_BPF_TYPE_HASH,
-    [BF_SET_SRCIP6PORT] = BF_MAP_BPF_TYPE_HASH,
-    [BF_SET_SRCIP6] = BF_MAP_BPF_TYPE_HASH,
-    [BF_SET_IP4_SUBNET] = BF_MAP_BPF_TYPE_LPM_TRIE,
-    [BF_SET_IP6_SUBNET] = BF_MAP_BPF_TYPE_LPM_TRIE,
-};
-
-static_assert(ARRAY_SIZE(_bf_map_bpf_type_to_set_type) == _BF_SET_MAX, "");
-
-enum bf_map_bpf_type bf_map_bpf_type_from_set_type(enum bf_set_type type)
-{
-    return _bf_map_bpf_type_to_set_type[type];
+    return bf_bpf_map_update_elem(map->fd, key, value, 0);
 }
