@@ -10,17 +10,10 @@
 #include <stddef.h>
 #include <stdint.h>
 
-#include "core/dump.h"
-#include "core/set.h"
-
-enum bf_map_bpf_type
-{
-    BF_MAP_BPF_TYPE_ARRAY,
-    BF_MAP_BPF_TYPE_HASH,
-    BF_MAP_BPF_TYPE_LPM_TRIE,
-    BF_MAP_BPF_TYPE_RINGBUF,
-    _BF_MAP_BPF_TYPE_MAX,
-};
+#include <bpfilter/bpf_types.h>
+#include <bpfilter/dump.h>
+#include <bpfilter/pack.h>
+#include <bpfilter/set.h>
 
 enum bf_map_type
 {
@@ -40,28 +33,16 @@ enum bf_map_type
 struct bf_map
 {
     enum bf_map_type type;
-    int fd;
+    enum bf_bpf_map_type bpf_type;
     char name[BPF_OBJ_NAME_LEN];
-    enum bf_map_bpf_type bpf_type;
     size_t key_size;
     size_t value_size;
     size_t n_elems;
+    int fd;
 };
 
-struct bf_marsh;
-
 #define _free_bf_map_ __attribute__((__cleanup__(bf_map_free)))
 
-/**
- * Convenience macro to initialize a list of @ref bf_map .
- *
- * @return An initialized @ref bf_list that can contain @ref bf_map object,
- *         with its @ref bf_list_ops properly configured.
- */
-#define bf_map_list()                                                          \
-    ((bf_list) {.ops = {.free = (bf_list_ops_free)bf_map_free,                 \
-                        .marsh = (bf_list_ops_marsh)bf_map_marsh}})
-
 /**
  * Allocates and initializes a new BPF map object.
  *
@@ -69,14 +50,12 @@ struct bf_marsh;
  * object used to keep track of a BPF map on the system.
  *
  * @param map BPF map object to allocate and initialize. Can't be NULL. On
- *            success, @c *map points to a valid @ref bf_map . On failure,
- *            @c *map remain unchanged.
+ *        success, @c *map points to a valid @ref bf_map . On failure,
+ *        @c *map remain unchanged.
  * @param name Name of the map. Will be used as the name of the BPF object, but
  *        also as filename when pinning the map to the system. Can't be NULL or
  *        empty.
  * @param type Map type, defines the set of available operations.
- * @param bpf_type Map type. Not all BPF maps are supported, see
- *        @ref bf_map_bpf_type for the full list of supported types.
  * @param key_size Size (in bytes) of a key in the map.
  * @param value_size Size (in bytes) of an element of the map.
  * @param n_elems Number of elements to reserve room for in the map. Can't be 0.
@@ -86,26 +65,38 @@ struct bf_marsh;
  * @return 0 on success, or a negative errno value on error.
  */
 int bf_map_new(struct bf_map **map, const char *name, enum bf_map_type type,
-               enum bf_map_bpf_type bpf_type, size_t key_size,
-               size_t value_size, size_t n_elems);
+               size_t key_size, size_t value_size, size_t n_elems);
 
 /**
- * Create a new BPF map object from serialized data.
+ * @brief Allocate and initialise a new BPF map object, from a set.
  *
- * @note The new BPF map object will represent a BPF map from bpfilter's point
+ * @param map BPF map object to allocate and initialize. Can't be NULL. On
+ *        success, @c *map points to a valid @ref bf_map . On failure, @c *map
+ *        remain unchanged. Can't be NULL.
+ * @param name Name of the map. Will be used as the name of the BPF object, but
+ *        also as filename when pinning the map to the system. Can't be NULL or
+ *        empty.
+ * @param set Set to create the map from. Can't be NULL.
+ * @return 0 on success, or a negative error value on error.
+ */
+int bf_map_new_from_set(struct bf_map **map, const char *name,
+                        const struct bf_set *set);
+
+/**
+ * @brief Allocate and initialize a new map from serialized data.
+ *
+ * @note The new bf_map object will represent a BPF map from bpfilter's point
  * of view, but it's not a BPF map.
  *
- * @param map BPF map object to allocate and initialize from the serialized
- *            data. The caller will own the object. On success, @c *map points
- *            to a valid BPF object map. On failure, @c *map is unchanged.
- *            Can't be NULL.
+ * @param map Map object to allocate and initialize from the serialized data.
+ *        The caller will own the object. On failure, `*map` is unchanged.
+ *        Can't be NULL.
  * @param dir_fd File descriptor of the directory containing the map's pin.
- *        Must be a valid file descriptor, or 0 is the map should not be opened.
- * @param marsh Serialized BPF map object data. Can't be NULL.
+ *        Must be a valid file descriptor, or -1 is the pin should not be opened.
+ * @param node Node containing the serialized map. Can't be NULL.
  * @return 0 on success, or a negative errno value on failure.
  */
-int bf_map_new_from_marsh(struct bf_map **map, int dir_fd,
-                          const struct bf_marsh *marsh);
+int bf_map_new_from_pack(struct bf_map **map, int dir_fd, bf_rpack_node_t node);
 
 /**
  * Free a BPF map object.
@@ -119,17 +110,13 @@ int bf_map_new_from_marsh(struct bf_map **map, int dir_fd,
 void bf_map_free(struct bf_map **map);
 
 /**
- * Serializes a BPF map object.
+ * @brief Serialize a map.
  *
- * @param map BPF map object to serialize. The object itself won't be modified.
- *            Can't be NULL.
- * @param marsh Marsh object, will be allocated by this function and owned by
- *              the caller. On success, @c *marsh will point to the BPF map's
- *              serialized data. On failure, @c *marsh is left unchanged. Can't
- *              be NULL.
- * @return 0 on success, or a negative errno value on error.
+ * @param map Map to serialize. Can't be NULL.
+ * @param pack `bf_wpack_t` object to serialize the map into. Can't be NULL.
+ * @return 0 on success, or a negative error value on failure.
  */
-int bf_map_marsh(const struct bf_map *map, struct bf_marsh **marsh);
+int bf_map_pack(const struct bf_map *map, bf_wpack_t *pack);
 
 /**
  * Dump a BPF map object.
@@ -144,11 +131,9 @@ void bf_map_dump(const struct bf_map *map, prefix_t *prefix);
  * Create the BPF map.
  *
  * @param map BPF map to create. Can't be NULL.
- * @param flags Flags to use during map creation. All the flags supported by
- *              @c BPF_MAP_CREATE can be used.
  * @return 0 on success, or a negative errno value on failure.
  */
-int bf_map_create(struct bf_map *map, uint32_t flags);
+int bf_map_create(struct bf_map *map);
 
 /**
  * Destroy the BPF map.
@@ -233,33 +218,3 @@ int bf_map_set_n_elems(struct bf_map *map, size_t n_elems);
  * @return 0 on success, or a negative errno value on failure.
  */
 int bf_map_set_elem(const struct bf_map *map, void *key, void *value);
-
-/**
- * Convert a @ref bf_map_bpf_type to a string.
- *
- * @param bpf_type Map type to convert to string. Must be a valid
- *        @ref bf_map_bpf_type (except for @ref _BF_MAP_BPF_TYPE_MAX ).
- * @return The map type, as a string.
- */
-const char *bf_map_bpf_type_to_str(enum bf_map_bpf_type bpf_type);
-
-/**
- * Convert a string into a @ref bf_map_bpf_type value.
- *
- * If the string is an invalid @ref bf_map_bpf_type string representation,
- * an error is returned.
- *
- * @param str String to convert to a @ref bf_map_bpf_type value. Can't be NULL.
- * @param bpf_type On success, contains the map type value. Unchanged on failure.
- *        Can't be NULL.
- * @return 0 on success, or a negative errno value on failure.
- */
-int bf_map_bpf_type_from_str(const char *str, enum bf_map_bpf_type *bpf_type);
-
-/**
- * @brief Convert a generic set type into a BPF map type.
- *
- * @param type Set type to convert.
- * @return BPF map type corresponding to the set type.
- */
-enum bf_map_bpf_type bf_map_bpf_type_from_set_type(enum bf_set_type type);
