@@ -3,7 +3,7 @@
  * Copyright (c) 2023 Meta Platforms, Inc. and affiliates.
  */
 
-#include "bpfilter/cgen/program.h"
+#include "cgen/program.h"
 
 #include <linux/bpf.h>
 #include <linux/bpf_common.h>
@@ -19,50 +19,51 @@
 #include <string.h>
 #include <unistd.h>
 
-#include "bpfilter/cgen/cgroup.h"
-#include "bpfilter/cgen/dump.h"
-#include "bpfilter/cgen/fixup.h"
-#include "bpfilter/cgen/jmp.h"
-#include "bpfilter/cgen/matcher/icmp.h"
-#include "bpfilter/cgen/matcher/ip4.h"
-#include "bpfilter/cgen/matcher/ip6.h"
-#include "bpfilter/cgen/matcher/meta.h"
-#include "bpfilter/cgen/matcher/set.h"
-#include "bpfilter/cgen/matcher/tcp.h"
-#include "bpfilter/cgen/matcher/udp.h"
-#include "bpfilter/cgen/nf.h"
-#include "bpfilter/cgen/printer.h"
-#include "bpfilter/cgen/prog/link.h"
-#include "bpfilter/cgen/prog/map.h"
-#include "bpfilter/cgen/stub.h"
-#include "bpfilter/cgen/tc.h"
-#include "bpfilter/cgen/xdp.h"
-#include "bpfilter/ctx.h"
-#include "bpfilter/opts.h"
-#include "core/bpf.h"
-#include "core/btf.h"
-#include "core/chain.h"
-#include "core/counter.h"
-#include "core/dump.h"
-#include "core/flavor.h"
-#include "core/helper.h"
-#include "core/hook.h"
-#include "core/io.h"
-#include "core/list.h"
-#include "core/logger.h"
-#include "core/marsh.h"
-#include "core/matcher.h"
-#include "core/rule.h"
-#include "core/set.h"
-#include "core/verdict.h"
+#include <bpfilter/bpf.h>
+#include <bpfilter/btf.h>
+#include <bpfilter/chain.h>
+#include <bpfilter/counter.h>
+#include <bpfilter/dump.h>
+#include <bpfilter/flavor.h>
+#include <bpfilter/helper.h>
+#include <bpfilter/hook.h>
+#include <bpfilter/io.h>
+#include <bpfilter/list.h>
+#include <bpfilter/logger.h>
+#include <bpfilter/matcher.h>
+#include <bpfilter/pack.h>
+#include <bpfilter/rule.h>
+#include <bpfilter/set.h>
+#include <bpfilter/verdict.h>
 
-#include "external/filter.h"
+#include "cgen/cgroup.h"
+#include "cgen/dump.h"
+#include "cgen/fixup.h"
+#include "cgen/jmp.h"
+#include "cgen/matcher/icmp.h"
+#include "cgen/matcher/ip4.h"
+#include "cgen/matcher/ip6.h"
+#include "cgen/matcher/meta.h"
+#include "cgen/matcher/set.h"
+#include "cgen/matcher/tcp.h"
+#include "cgen/matcher/udp.h"
+#include "cgen/nf.h"
+#include "cgen/printer.h"
+#include "cgen/prog/link.h"
+#include "cgen/prog/map.h"
+#include "cgen/stub.h"
+#include "cgen/tc.h"
+#include "cgen/xdp.h"
+#include "ctx.h"
+#include "filter.h"
+#include "opts.h"
 
 #define _BF_LOG_BUF_SIZE                                                       \
     (UINT32_MAX >> 8) /* verifier maximum in kernels <= 5.1 */
 #define _BF_PROGRAM_DEFAULT_IMG_SIZE (1 << 6)
 #define _BF_LOG_MAP_N_ENTRIES 1000
-#define _BF_LOG_MAP_SIZE _bf_round_next_power_of_2(sizeof(struct bf_log) * _BF_LOG_MAP_N_ENTRIES)
+#define _BF_LOG_MAP_SIZE                                                       \
+    _bf_round_next_power_of_2(sizeof(struct bf_log) * _BF_LOG_MAP_N_ENTRIES)
 
 static inline size_t _bf_round_next_power_of_2(size_t value)
 {
@@ -112,31 +113,27 @@ int bf_program_new(struct bf_program **program, const struct bf_chain *chain)
     (void)snprintf(_program->prog_name, BPF_OBJ_NAME_LEN, "%s", "bf_prog");
 
     r = bf_map_new(&_program->cmap, "counters_map", BF_MAP_TYPE_COUNTERS,
-                   BF_MAP_BPF_TYPE_ARRAY, sizeof(uint32_t),
-                   sizeof(struct bf_counter), 1);
+                   sizeof(uint32_t), sizeof(struct bf_counter), 1);
     if (r < 0)
         return bf_err_r(r, "failed to create the counters bf_map object");
 
     r = bf_map_new(&_program->pmap, "printer_map", BF_MAP_TYPE_PRINTER,
-                   BF_MAP_BPF_TYPE_ARRAY, sizeof(uint32_t),
-                   BF_MAP_VALUE_SIZE_UNKNOWN, 1);
+                   sizeof(uint32_t), BF_MAP_VALUE_SIZE_UNKNOWN, 1);
     if (r < 0)
         return bf_err_r(r, "failed to create the printer bf_map object");
 
-    r = bf_map_new(&_program->lmap, "log_map", BF_MAP_TYPE_LOG,
-                   BF_MAP_BPF_TYPE_RINGBUF, 0, 0, _BF_LOG_MAP_SIZE);
+    r = bf_map_new(&_program->lmap, "log_map", BF_MAP_TYPE_LOG, 0, 0,
+                   _BF_LOG_MAP_SIZE);
     if (r < 0)
         return bf_err_r(r, "failed to create the log bf_map object");
 
-    _program->sets = bf_map_list();
+    _program->sets = bf_list_default(bf_map_free, bf_map_pack);
     bf_list_foreach (&chain->sets, set_node) {
         struct bf_set *set = bf_list_node_get_data(set_node);
         _free_bf_map_ struct bf_map *map = NULL;
 
         (void)snprintf(name, BPF_OBJ_NAME_LEN, "set_%04x", (uint8_t)set_idx++);
-        r = bf_map_new(&map, name, BF_MAP_TYPE_SET,
-                       bf_map_bpf_type_from_set_type(set->type), set->elem_size,
-                       1, bf_list_size(&set->elems));
+        r = bf_map_new_from_set(&map, name, set);
         if (r < 0)
             return r;
 
@@ -162,6 +159,100 @@ int bf_program_new(struct bf_program **program, const struct bf_chain *chain)
     return 0;
 }
 
+int bf_program_new_from_pack(struct bf_program **program,
+                             const struct bf_chain *chain, int dir_fd,
+                             bf_rpack_node_t node)
+{
+    _free_bf_program_ struct bf_program *_program = NULL;
+    _free_bf_link_ struct bf_link *link = NULL;
+    const void *img;
+    size_t img_len;
+    bf_rpack_node_t child, array_node;
+    int r;
+
+    bf_assert(program);
+    bf_assert(chain);
+
+    r = bf_program_new(&_program, chain);
+    if (r < 0)
+        return r;
+
+    bf_map_free(&_program->cmap);
+    r = bf_rpack_kv_obj(node, "cmap", &child);
+    if (r)
+        return bf_rpack_key_err(r, "bf_program.cmap");
+    r = bf_map_new_from_pack(&_program->cmap, dir_fd, child);
+    if (r)
+        return r;
+
+    bf_map_free(&_program->pmap);
+    r = bf_rpack_kv_obj(node, "pmap", &child);
+    if (r)
+        return bf_rpack_key_err(r, "bf_program.pmap");
+    r = bf_map_new_from_pack(&_program->pmap, dir_fd, child);
+    if (r)
+        return r;
+
+    bf_map_free(&_program->lmap);
+    r = bf_rpack_kv_obj(node, "lmap", &child);
+    if (r)
+        return bf_rpack_key_err(r, "bf_program.lmap");
+    r = bf_map_new_from_pack(&_program->lmap, dir_fd, child);
+    if (r)
+        return r;
+
+    bf_list_clean(&_program->sets);
+    _program->sets = bf_list_default(bf_map_free, bf_map_pack);
+    r = bf_rpack_kv_array(node, "sets", &child);
+    if (r)
+        return bf_rpack_key_err(r, "bf_program.sets");
+    bf_rpack_array_foreach (child, array_node) {
+        _free_bf_map_ struct bf_map *map = NULL;
+
+        r = bf_list_emplace(&_program->sets, bf_map_new_from_pack, map, dir_fd,
+                            array_node);
+        if (r)
+            return bf_err_r(r, "failed to unpack bf_map into bf_program.sets");
+    }
+
+    /* Try to restore the link: on success, replace the program's link with the
+     * restored on. If -ENOENT is returned, the link doesn't exist, meaning the
+     * program is not attached. Otherwise, return an error. */
+    r = bf_rpack_kv_obj(node, "link", &child);
+    if (r)
+        return bf_rpack_key_err(r, "bf_program.link");
+    r = bf_link_new_from_pack(&link, dir_fd, child);
+    if (!r)
+        bf_swap(_program->link, link);
+    else if (r != -ENOENT)
+        return bf_err_r(r, "failed to restore bf_program.link");
+
+    bf_printer_free(&_program->printer);
+    r = bf_rpack_kv_obj(node, "printer", &child);
+    if (r)
+        return bf_rpack_key_err(r, "bf_program.printer");
+    r = bf_printer_new_from_pack(&_program->printer, child);
+    if (r)
+        return r;
+
+    r = bf_rpack_kv_bin(node, "img", &img, &img_len);
+    if (r)
+        return bf_rpack_key_err(r, "bf_program.img");
+    _program->img = bf_memdup(img, img_len);
+    if (!_program->img)
+        return bf_rpack_key_err(-ENOMEM, "bf_program.img");
+    _program->img_size = img_len / sizeof(struct bpf_insn);
+    _program->img_cap = _program->img_size;
+
+    r = bf_bpf_obj_get(_program->prog_name, dir_fd, &_program->runtime.prog_fd);
+    if (r < 0)
+        return bf_err_r(r, "failed to restore bf_program.fd");
+
+    *program = TAKE_PTR(_program);
+
+    return 0;
+}
+
 void bf_program_free(struct bf_program **program)
 {
     if (!*program)
@@ -188,215 +279,37 @@ void bf_program_free(struct bf_program **program)
     *program = NULL;
 }
 
-int bf_program_marsh(const struct bf_program *program, struct bf_marsh **marsh)
+int bf_program_pack(const struct bf_program *program, bf_wpack_t *pack)
 {
-    _free_bf_marsh_ struct bf_marsh *_marsh = NULL;
-    int r;
-
     bf_assert(program);
-    bf_assert(marsh);
+    bf_assert(pack);
 
-    r = bf_marsh_new(&_marsh, NULL, 0);
-    if (r < 0)
-        return r;
+    bf_wpack_open_object(pack, "cmap");
+    bf_map_pack(program->cmap, pack);
+    bf_wpack_close_object(pack);
 
-    {
-        // Serialize bf_program.counters
-        _free_bf_marsh_ struct bf_marsh *counters_elem = NULL;
+    bf_wpack_open_object(pack, "pmap");
+    bf_map_pack(program->pmap, pack);
+    bf_wpack_close_object(pack);
 
-        r = bf_map_marsh(program->cmap, &counters_elem);
-        if (r < 0)
-            return r;
+    bf_wpack_open_object(pack, "lmap");
+    bf_map_pack(program->lmap, pack);
+    bf_wpack_close_object(pack);
 
-        r = bf_marsh_add_child_obj(&_marsh, counters_elem);
-        if (r < 0)
-            return r;
-    }
+    bf_wpack_kv_list(pack, "sets", &program->sets);
 
-    {
-        // Serialize bf_program.pmap
-        _free_bf_marsh_ struct bf_marsh *pmap_elem = NULL;
+    bf_wpack_open_object(pack, "link");
+    bf_link_pack(program->link, pack);
+    bf_wpack_close_object(pack);
 
-        r = bf_map_marsh(program->pmap, &pmap_elem);
-        if (r < 0)
-            return r;
+    bf_wpack_open_object(pack, "printer");
+    bf_printer_pack(program->printer, pack);
+    bf_wpack_close_object(pack);
 
-        r = bf_marsh_add_child_obj(&_marsh, pmap_elem);
-        if (r < 0)
-            return r;
-    }
+    bf_wpack_kv_bin(pack, "img", program->img,
+                    program->img_size * sizeof(struct bpf_insn));
 
-    {
-        // Serialize bf_program.lmap
-        _free_bf_marsh_ struct bf_marsh *lmap_elem = NULL;
-
-        r = bf_map_marsh(program->lmap, &lmap_elem);
-        if (r < 0)
-            return r;
-
-        r = bf_marsh_add_child_obj(&_marsh, lmap_elem);
-        if (r < 0)
-            return r;
-    }
-
-    {
-        // Serialize bf_program.sets
-        _free_bf_marsh_ struct bf_marsh *sets_elem = NULL;
-
-        r = bf_list_marsh(&program->sets, &sets_elem);
-        if (r < 0)
-            return r;
-
-        r = bf_marsh_add_child_obj(&_marsh, sets_elem);
-        if (r < 0) {
-            return bf_err_r(
-                r,
-                "failed to insert serialized sets into bf_program serialized data");
-        }
-    }
-
-    {
-        // Serialize bf_program.links
-        _free_bf_marsh_ struct bf_marsh *links_elem = NULL;
-
-        r = bf_link_marsh(program->link, &links_elem);
-        if (r)
-            return bf_err_r(r, "failed to serialize bf_program.link");
-
-        r = bf_marsh_add_child_obj(&_marsh, links_elem);
-        if (r) {
-            return bf_err_r(
-                r,
-                "failed to insert serialized link into bf_program serialized data");
-        }
-    }
-
-    {
-        // Serialise bf_program.printer
-        _free_bf_marsh_ struct bf_marsh *child = NULL;
-
-        r = bf_printer_marsh(program->printer, &child);
-        if (r)
-            return bf_err_r(r, "failed to marsh bf_printer object");
-
-        r = bf_marsh_add_child_obj(&_marsh, child);
-        if (r)
-            return bf_err_r(r, "failed to append object to marsh");
-    }
-
-    r |= bf_marsh_add_child_raw(&_marsh, program->img,
-                                program->img_size * sizeof(struct bpf_insn));
-    if (r)
-        return bf_err_r(r, "Failed to serialize program");
-
-    *marsh = TAKE_PTR(_marsh);
-
-    return 0;
-}
-
-int bf_program_unmarsh(const struct bf_marsh *marsh,
-                       struct bf_program **program,
-                       const struct bf_chain *chain, int dir_fd)
-{
-    _free_bf_program_ struct bf_program *_program = NULL;
-    _free_bf_link_ struct bf_link *link = NULL;
-    struct bf_marsh *child = NULL;
-    int r;
-
-    bf_assert(marsh && program);
-
-    r = bf_program_new(&_program, chain);
-    if (r < 0)
-        return r;
-
-    if (!(child = bf_marsh_next_child(marsh, child)))
-        return -EINVAL;
-    bf_map_free(&_program->cmap);
-    r = bf_map_new_from_marsh(&_program->cmap, dir_fd, child);
-    if (r < 0)
-        return r;
-
-    if (!(child = bf_marsh_next_child(marsh, child)))
-        return -EINVAL;
-    bf_map_free(&_program->pmap);
-    r = bf_map_new_from_marsh(&_program->pmap, dir_fd, child);
-    if (r < 0)
-        return r;
-
-    if (!(child = bf_marsh_next_child(marsh, child)))
-        return -EINVAL;
-    bf_map_free(&_program->lmap);
-    r = bf_map_new_from_marsh(&_program->lmap, dir_fd, child);
-    if (r < 0)
-        return r;
-
-    /** @todo Avoid creating and filling the list in @ref bf_program_new before
-     * trashing it all here. Eventually, this function will be replaced with
-     * @c bf_program_new_from_marsh and this issue could be solved by **not**
-     * relying on @ref bf_program_new to allocate an initialize @p _program . */
-    bf_list_clean(&_program->sets);
-    _program->sets = bf_map_list();
-
-    if (!(child = bf_marsh_next_child(marsh, child)))
-        return -EINVAL;
-    {
-        // Unmarsh bf_program.sets
-        struct bf_marsh *set_elem = NULL;
-
-        while ((set_elem = bf_marsh_next_child(child, set_elem))) {
-            _free_bf_map_ struct bf_map *map = NULL;
-
-            r = bf_map_new_from_marsh(&map, dir_fd, set_elem);
-            if (r < 0)
-                return r;
-
-            r = bf_list_add_tail(&_program->sets, map);
-            if (r < 0)
-                return r;
-
-            TAKE_PTR(map);
-        }
-    }
-
-    // Unmarsh bf_program.links
-    if (!(child = bf_marsh_next_child(marsh, child)))
-        return -EINVAL;
-
-    /* Try to restore the link: on success, replace the program's link with the
-     * restored on. If -ENOENT is returned, the link doesn't exist, meaning the
-     * program is not attached. Otherwise, return an error. */
-    r = bf_link_new_from_marsh(&link, dir_fd, child);
-    if (!r)
-        bf_swap(_program->link, link);
-    else if (r != -ENOENT)
-        return bf_err_r(r, "failed to restore bf_program.link");
-
-    // Unmarsh bf_program.printer
-    child = bf_marsh_next_child(marsh, child);
-    if (!child)
-        return bf_err_r(-EINVAL, "failed to find valid child");
-
-    bf_printer_free(&_program->printer);
-    r = bf_printer_new_from_marsh(&_program->printer, child);
-    if (r)
-        return bf_err_r(r, "failed to restore bf_printer object");
-
-    if (!(child = bf_marsh_next_child(marsh, child)))
-        return -EINVAL;
-    _program->img = bf_memdup(child->data, child->data_len);
-    _program->img_size = child->data_len / sizeof(struct bpf_insn);
-    _program->img_cap = child->data_len / sizeof(struct bpf_insn);
-
-    if (bf_marsh_next_child(marsh, child))
-        bf_warn("codegen marsh has more children than expected");
-
-    r = bf_bpf_obj_get(_program->prog_name, dir_fd, &_program->runtime.prog_fd);
-    if (r < 0)
-        return bf_err_r(r, "failed to get prog fd");
-
-    *program = TAKE_PTR(_program);
-
-    return 0;
+    return bf_wpack_is_valid(pack) ? 0 : -EINVAL;
 }
 
 void bf_program_dump(const struct bf_program *program, prefix_t *prefix)
@@ -590,13 +503,14 @@ static int _bf_program_generate_rule(struct bf_program *program,
     bf_list_foreach (&rule->matchers, matcher_node) {
         struct bf_matcher *matcher = bf_list_node_get_data(matcher_node);
 
-        switch (matcher->type) {
+        switch (bf_matcher_get_type(matcher)) {
         case BF_MATCHER_META_IFACE:
         case BF_MATCHER_META_L3_PROTO:
         case BF_MATCHER_META_L4_PROTO:
         case BF_MATCHER_META_PROBABILITY:
         case BF_MATCHER_META_SPORT:
         case BF_MATCHER_META_DPORT:
+        case BF_MATCHER_META_MARK:
             r = bf_matcher_generate_meta(program, matcher);
             if (r)
                 return r;
@@ -632,12 +546,6 @@ static int _bf_program_generate_rule(struct bf_program *program,
             if (r)
                 return r;
             break;
-        case BF_MATCHER_SET_SRCIP6PORT:
-        case BF_MATCHER_SET_SRCIP6:
-            r = bf_matcher_generate_set(program, matcher);
-            if (r)
-                return r;
-            break;
         case BF_MATCHER_ICMP_TYPE:
         case BF_MATCHER_ICMP_CODE:
         case BF_MATCHER_ICMPV6_TYPE:
@@ -646,15 +554,36 @@ static int _bf_program_generate_rule(struct bf_program *program,
             if (r)
                 return r;
             break;
+        case BF_MATCHER_SET:
+            r = bf_matcher_generate_set(program, matcher);
+            if (r)
+                return r;
+            break;
         default:
-            return bf_err_r(-EINVAL, "unknown matcher type %d", matcher->type);
+            return bf_err_r(-EINVAL, "unknown matcher type %d",
+                            bf_matcher_get_type(matcher));
         };
     }
 
+    if (bf_rule_mark_is_set(rule)) {
+        if (!program->runtime.ops->gen_inline_set_mark) {
+            return bf_err_r(-ENOTSUP, "set mark is not supported by %s",
+                            program->runtime.chain->name);
+        }
+
+        r = program->runtime.ops->gen_inline_set_mark(program,
+                                                      bf_rule_mark_get(rule));
+        if (r) {
+            return bf_err_r(r,
+                            "failed to generate bytecode to set mark for '%s'",
+                            program->runtime.chain->name);
+        }
+    }
+
     if (rule->log) {
         EMIT(program, BPF_MOV64_REG(BPF_REG_1, BPF_REG_10));
         EMIT(program, BPF_ALU64_IMM(BPF_ADD, BPF_REG_1, BF_PROG_CTX_OFF(arg)));
-        EMIT_LOAD_LOG_FD_FIXUP(program, BPF_REG_2);
+        EMIT(program, BPF_MOV64_IMM(BPF_REG_2, rule->index));
         EMIT(program, BPF_MOV64_IMM(BPF_REG_3, rule->log));
         EMIT(program, BPF_MOV64_REG(BPF_REG_4, BPF_REG_7));
         EMIT(program, BPF_MOV64_REG(BPF_REG_5, BPF_REG_8));
@@ -877,19 +806,17 @@ int bf_program_generate(struct bf_program *program)
     EMIT(program, BPF_MOV64_IMM(BPF_REG_7, 0));
     EMIT(program, BPF_MOV64_IMM(BPF_REG_8, 0));
 
-    program->ipv6_nexthdr = false;
-    bf_list_foreach (&chain->rules, rule_node) {
-        struct bf_rule *rule = bf_list_node_get_data(rule_node);
-        bf_list_foreach (&rule->matchers, matcher_node) {
-            struct bf_matcher *matcher = bf_list_node_get_data(matcher_node);
-            if (matcher->type == BF_MATCHER_IP6_NEXTHDR) {
-                // Zeroing IPv6 extension headers
-                EMIT(program, BPF_STX_MEM(BPF_DW, BPF_REG_10, BPF_REG_7,
-                                          BF_PROG_CTX_OFF(ipv6_eh)));
-                program->ipv6_nexthdr = true;
-                break;
-            }
-        }
+    // If at least one rule logs the matched packets, populate ctx->log_map
+    if (program->runtime.chain->flags & BF_FLAG(BF_CHAIN_LOG)) {
+        EMIT_LOAD_LOG_FD_FIXUP(program, BPF_REG_2);
+        EMIT(program, BPF_STX_MEM(BPF_DW, BPF_REG_10, BPF_REG_2,
+                                  BF_PROG_CTX_OFF(log_map)));
+    }
+
+    // Zeroing IPv6 extension headers
+    if (program->runtime.chain->flags & BF_FLAG(BF_CHAIN_STORE_NEXTHDR)) {
+        EMIT(program, BPF_STX_MEM(BPF_DW, BPF_REG_10, BPF_REG_7,
+                                  BF_PROG_CTX_OFF(ipv6_eh)));
     }
 
     r = program->runtime.ops->gen_inline_prologue(program);
@@ -908,6 +835,7 @@ int bf_program_generate(struct bf_program *program)
         return r;
 
     // Call the update counters function
+    /// @todo Allow chains to have no counters at all.
     EMIT(program, BPF_MOV64_REG(BPF_REG_1, BPF_REG_10));
     EMIT(program, BPF_ALU64_IMM(BPF_ADD, BPF_REG_1, BF_PROG_CTX_OFF(arg)));
     EMIT_LOAD_COUNTERS_FD_FIXUP(program, BPF_REG_2);
@@ -947,7 +875,7 @@ static int _bf_program_load_printer_map(struct bf_program *program)
     if (r < 0)
         return r;
 
-    r = bf_map_create(program->pmap, 0);
+    r = bf_map_create(program->pmap);
     if (r < 0)
         return r;
 
@@ -976,7 +904,7 @@ static int _bf_program_load_counters_map(struct bf_program *program)
     if (r < 0)
         return r;
 
-    r = bf_map_create(program->cmap, 0);
+    r = bf_map_create(program->cmap);
     if (r < 0)
         return r;
 
@@ -996,7 +924,7 @@ static int _bf_program_load_log_map(struct bf_program *program)
 
     bf_assert(program);
 
-    r = bf_map_create(program->lmap, 0);
+    r = bf_map_create(program->lmap);
     if (r < 0)
         return r;
 
@@ -1027,10 +955,9 @@ static int _bf_program_load_sets_maps(struct bf_program *new_prog)
         struct bf_set *set = bf_list_node_get_data(set_node);
         struct bf_map *map = bf_list_node_get_data(map_node);
         size_t nelems = bf_list_size(&set->elems);
-        union bpf_attr attr = {};
         size_t idx = 0;
 
-        r = bf_map_create(map, 0);
+        r = bf_map_create(map);
         if (r < 0) {
             r = bf_err_r(r, "failed to create BPF map for set");
             goto err_destroy_maps;
@@ -1056,13 +983,7 @@ static int _bf_program_load_sets_maps(struct bf_program *new_prog)
             ++idx;
         }
 
-        attr.batch.map_fd = map->fd;
-        attr.batch.keys = (unsigned long long)keys;
-        attr.batch.values = (unsigned long long)values;
-        attr.batch.count = nelems;
-        attr.batch.flags = BPF_ANY;
-
-        r = bf_bpf(BPF_MAP_UPDATE_BATCH, &attr);
+        r = bf_bpf_map_update_batch(map->fd, keys, values, nelems, BPF_ANY);
         if (r < 0) {
             bf_err_r(r, "failed to add set elements to the map");
             goto err_destroy_maps;
