@@ -8,17 +8,18 @@
 #include <stddef.h>
 #include <stdint.h>
 
-#include "bpfilter/cgen/elfstub.h"
-#include "bpfilter/cgen/fixup.h"
-#include "bpfilter/cgen/printer.h"
-#include "bpfilter/cgen/runtime.h"
-#include "core/chain.h"
-#include "core/dump.h"
-#include "core/flavor.h"
-#include "core/helper.h"
-#include "core/list.h"
+#include <bpfilter/chain.h>
+#include <bpfilter/dump.h>
+#include <bpfilter/flavor.h>
+#include <bpfilter/helper.h>
+#include <bpfilter/list.h>
+#include <bpfilter/pack.h>
 
-#include "external/filter.h"
+#include "cgen/elfstub.h"
+#include "cgen/fixup.h"
+#include "cgen/printer.h"
+#include "cgen/runtime.h"
+#include "filter.h"
 
 /**
  * @file program.h
@@ -185,7 +186,6 @@
 
 struct bf_chain;
 struct bf_map;
-struct bf_marsh;
 struct bf_counter;
 struct bf_link;
 struct bf_hookopts;
@@ -207,10 +207,10 @@ struct bf_program
     /// List of set maps
     bf_list sets;
 
-    /// Link objects attaching the program to a hook.
+    /** Link objects attaching the program to a hook.
+     * @todo A ``bf_program`` should not have any link until the program is
+     * attached. */
     struct bf_link *link;
-    /// Program with ip6.nexthdr rules
-    bool ipv6_nexthdr;
 
     /* Bytecode */
     uint32_t elfstubs_location[_BF_ELFSTUB_MAX];
@@ -241,12 +241,48 @@ struct bf_program
 
 #define _free_bf_program_ __attribute__((__cleanup__(bf_program_free)))
 
+/**
+ * @brief Allocate and initialize a new `bf_program` object.
+ *
+ * @param program `bf_program` object to allocate and initialize. Can't be NULL.
+ * @param chain Chain the program is generated from. Can't be NULL.
+ * @return 0 on success, or a negative error value on failure.
+ */
 int bf_program_new(struct bf_program **program, const struct bf_chain *chain);
+
+/**
+ * @brief Allocate and initialize a new program from serialized data.
+ *
+ * @note The new bf_program object will represent a BPF map from bpfilter's
+ * point of view, but it's not a BPF program.
+ *
+ * @todo `bf_program` should be recreated from the current system state by
+ * using `BF_OBJ_INFO_BF_FD`, and not serialized.
+ *
+ * @param program Program object to allocate and initialize from the serialized
+ *        data. The caller will own the object. On failure, `*program` is
+ *        unchanged. Can't be NULL.
+ * @param chain Chain to restore the program for. Can't be NULL.
+ * @param dir_fd File descriptor of the directory containing the program's pins.
+ *        Must be a valid file descriptor, or -1 if the pin should not be opened.
+ * @param node Node containing the serialized program. Can't be NULL.
+ * @return 0 on success, or a negative errno value on failure.
+ */
+int bf_program_new_from_pack(struct bf_program **program,
+                             const struct bf_chain *chain, int dir_fd,
+                             bf_rpack_node_t node);
+
 void bf_program_free(struct bf_program **program);
-int bf_program_marsh(const struct bf_program *program, struct bf_marsh **marsh);
-int bf_program_unmarsh(const struct bf_marsh *marsh,
-                       struct bf_program **program,
-                       const struct bf_chain *chain, int dir_fd);
+
+/**
+ * @brief Serialize a program.
+ *
+ * @param program Program to serialize. Can't be NULL.
+ * @param pack `bf_wpack_t` object to serialize the program into. Can't be NULL.
+ * @return 0 on success, or a negative error value on failure.
+ */
+int bf_program_pack(const struct bf_program *program, bf_wpack_t *pack);
+
 void bf_program_dump(const struct bf_program *program, prefix_t *prefix);
 int bf_program_grow_img(struct bf_program *program);
 
