@@ -3,7 +3,7 @@
  * Copyright (c) 2023 Meta Platforms, Inc. and affiliates.
  */
 
-#include "bpfilter/cgen/stub.h"
+#include "cgen/stub.h"
 
 #include <linux/bpf.h>
 #include <linux/bpf_common.h>
@@ -20,17 +20,18 @@
 #include <endian.h>
 #include <stddef.h>
 
-#include "bpfilter/cgen/elfstub.h"
-#include "bpfilter/cgen/jmp.h"
-#include "bpfilter/cgen/printer.h"
-#include "bpfilter/cgen/program.h"
-#include "bpfilter/cgen/swich.h"
-#include "bpfilter/opts.h"
-#include "core/flavor.h"
-#include "core/helper.h"
-#include "core/verdict.h"
+#include <bpfilter/flavor.h>
+#include <bpfilter/helper.h>
+#include <bpfilter/matcher.h>
+#include <bpfilter/verdict.h>
 
-#include "external/filter.h"
+#include "cgen/elfstub.h"
+#include "cgen/jmp.h"
+#include "cgen/printer.h"
+#include "cgen/program.h"
+#include "cgen/swich.h"
+#include "filter.h"
+#include "opts.h"
 
 #define _BF_LOW_EH_BITMASK 0x1801800000000801ULL
 
@@ -275,9 +276,12 @@ int bf_stub_parse_l3_hdr(struct bf_program *program)
         // Process EH
         EMIT(program, BPF_MOV64_REG(BPF_REG_1, BPF_REG_10));
         EMIT(program, BPF_ALU64_IMM(BPF_ADD, BPF_REG_1, BF_PROG_CTX_OFF(arg)));
-        EMIT_FIXUP_ELFSTUB(program, program->ipv6_nexthdr ?
-                                        BF_ELFSTUB_PARSE_IPV6_NH :
-                                        BF_ELFSTUB_PARSE_IPV6_EH);
+        // If any rule filters on ipv6.nexthdr, store the EH in the runtime context
+        // during process, so we won't have to process the EH again.
+        if (program->runtime.chain->flags & BF_FLAG(BF_CHAIN_STORE_NEXTHDR))
+            EMIT_FIXUP_ELFSTUB(program, BF_ELFSTUB_PARSE_IPV6_NH);
+        else
+            EMIT_FIXUP_ELFSTUB(program, BF_ELFSTUB_PARSE_IPV6_EH);
         EMIT(program, BPF_MOV64_REG(BPF_REG_8, BPF_REG_0));
 
         ehjmp = bf_jmpctx_get(program, BPF_JMP_A(0));
@@ -369,3 +373,86 @@ int bf_stub_parse_l4_hdr(struct bf_program *program)
 
     return 0;
 }
+
+int bf_stub_rule_check_protocol(struct bf_program *program,
+                                const struct bf_matcher_meta *meta)
+{
+    bf_assert(program && meta);
+
+    switch (meta->layer) {
+    case BF_MATCHER_LAYER_3:
+        EMIT_FIXUP_JMP_NEXT_RULE(
+            program, BPF_JMP_IMM(BPF_JNE, BPF_REG_7,
+                                 htobe16((uint16_t)meta->hdr_id), 0));
+        break;
+    case BF_MATCHER_LAYER_4:
+        EMIT_FIXUP_JMP_NEXT_RULE(
+            program, BPF_JMP_IMM(BPF_JNE, BPF_REG_8, (uint8_t)meta->hdr_id, 0));
+        break;
+    default:
+        return bf_err_r(-EINVAL, "rule can't check for layer ID %d",
+                        meta->layer);
+    }
+
+    return 0;
+}
+
+int bf_stub_load_header(struct bf_program *program,
+                        const struct bf_matcher_meta *meta, int reg)
+{
+    bf_assert(program && meta);
+
+    switch (meta->layer) {
+    case BF_MATCHER_LAYER_3:
+        EMIT(program,
+             BPF_LDX_MEM(BPF_DW, reg, BPF_REG_10, BF_PROG_CTX_OFF(l3_hdr)));
+        break;
+    case BF_MATCHER_LAYER_4:
+        EMIT(program,
+             BPF_LDX_MEM(BPF_DW, reg, BPF_REG_10, BF_PROG_CTX_OFF(l4_hdr)));
+        break;
+    default:
+        return bf_err_r(-EINVAL,
+                        "layer ID %d is not a valid layer to load header for",
+                        meta->layer);
+    }
+
+    return 0;
+}
+
+int bf_stub_stx_payload(struct bf_program *program,
+                        const struct bf_matcher_meta *meta, size_t offset)
+{
+    bf_assert(program && meta);
+
+    size_t remaining_size = meta->hdr_payload_size;
+    size_t src_offset = 0;
+    size_t dst_offset = offset;
+
+    while (remaining_size) {
+        int bpf_size = BPF_B;
+        size_t copy_bytes = 1;
+
+        if (BF_ALIGNED_64(offset) && remaining_size >= 8) {
+            bpf_size = BPF_DW;
+            copy_bytes = 8;
+        } else if (BF_ALIGNED_32(offset) && remaining_size >= 4) {
+            bpf_size = BPF_W;
+            copy_bytes = 4;
+        } else if (BF_ALIGNED_16(offset) && remaining_size >= 2) {
+            bpf_size = BPF_H;
+            copy_bytes = 2;
+        }
+
+        EMIT(program, BPF_LDX_MEM(bpf_size, BPF_REG_1, BPF_REG_6,
+                                  meta->hdr_payload_offset + src_offset));
+        EMIT(program, BPF_STX_MEM(bpf_size, BPF_REG_10, BPF_REG_1,
+                                  BF_PROG_SCR_OFF(dst_offset)));
+
+        remaining_size -= copy_bytes;
+        src_offset += copy_bytes;
+        dst_offset += copy_bytes;
+    }
+
+    return 0;
+}
