@@ -13,20 +13,21 @@
 #include <sys/un.h>
 #include <unistd.h>
 
-#include "bpfilter/ctx.h"
-#include "bpfilter/opts.h"
-#include "bpfilter/xlate/front.h"
-#include "core/btf.h"
-#include "core/dump.h"
-#include "core/front.h"
-#include "core/helper.h"
-#include "core/io.h"
-#include "core/logger.h"
-#include "core/marsh.h"
-#include "core/ns.h"
-#include "core/request.h"
-#include "core/response.h"
-#include "version.h"
+#include <bpfilter/btf.h>
+#include <bpfilter/dump.h>
+#include <bpfilter/front.h>
+#include <bpfilter/helper.h>
+#include <bpfilter/io.h>
+#include <bpfilter/logger.h>
+#include <bpfilter/ns.h>
+#include <bpfilter/pack.h>
+#include <bpfilter/request.h>
+#include <bpfilter/response.h>
+#include <bpfilter/version.h>
+
+#include "ctx.h"
+#include "opts.h"
+#include "xlate/front.h"
 
 /**
  * Global flag to indicate whether the daemon should stop.
@@ -71,9 +72,10 @@ void _bf_sig_handler(int sig)
  */
 static int _bf_load(const char *path)
 {
-    _cleanup_free_ struct bf_marsh *marsh = NULL;
-    struct bf_marsh *child = NULL;
-    size_t len;
+    _free_bf_rpack_ bf_rpack_t *pack = NULL;
+    _cleanup_free_ void *data = NULL;
+    bf_rpack_node_t child, array_node;
+    size_t data_len;
     int r;
 
     bf_assert(path);
@@ -90,46 +92,37 @@ static int _bf_load(const char *path)
         return 0;
     }
 
-    r = bf_read_file(path, (void **)&marsh, &len);
+    r = bf_read_file(path, &data, &data_len);
     if (r < 0)
         return r;
 
-    if (len < sizeof(struct bf_marsh))
-        return bf_err_r(EIO, "marshalled data is invalid");
+    r = bf_rpack_new(&pack, data, data_len);
+    if (r)
+        return r;
 
-    if (bf_marsh_size(marsh) != len) {
-        return bf_err_r(
-            EINVAL, "conflicting marshalled data size: got %zu, expected %zu",
-            len, bf_marsh_size(marsh));
-    }
-
-    child = bf_marsh_next_child(marsh, child);
-    if (!child) {
-        return bf_err_r(-EINVAL,
-                        "expecting a child in main marshalled context");
-    }
+    r = bf_rpack_kv_obj(bf_rpack_root(pack), "ctx", &child);
+    if (r)
+        return r;
 
     r = bf_ctx_load(child);
     if (r < 0)
         return r;
 
-    for (int i = 0; i < _BF_FRONT_MAX; ++i) {
-        child = bf_marsh_next_child(marsh, child);
-        if (!child) {
-            bf_err(
-                "no marshalled context for %s. Skipping restoration of remaining front-specific context.",
-                bf_front_to_str(i));
-            break;
-        }
+    r = bf_rpack_kv_array(bf_rpack_root(pack), "cache", &child);
+    if (r)
+        return r;
+    bf_rpack_array_foreach (child, array_node) {
+        if (bf_rpack_is_nil(array_node))
+            continue;
 
-        r = bf_front_ops_get(i)->unmarsh(child);
+        r = bf_front_ops_get(i)->unpack(array_node);
         if (r < 0) {
             return bf_err_r(r, "failed to restore context for %s",
                             bf_front_to_str(i));
         }
     }
 
-    bf_dbg("loaded marshalled context from %s", path);
+    bf_dbg("loaded serialized context from %s", path);
 
     return 1;
 }
@@ -137,14 +130,14 @@ static int _bf_load(const char *path)
 /**
  * Save bpfilter's runtime context to disk.
  *
- * Marshel the daemon's runtime context and save it to @p path.
- *
  * @param path Path to the context file.
  * @return 0 on success, negative error code on failure.
  */
 static int _bf_save(const char *path)
 {
-    _cleanup_free_ struct bf_marsh *marsh = NULL;
+    _free_bf_wpack_ bf_wpack_t *pack = NULL;
+    const void *data;
+    size_t data_len;
     int r;
 
     bf_assert(path);
@@ -156,46 +149,41 @@ static int _bf_save(const char *path)
         return 0;
     }
 
-    r = bf_marsh_new(&marsh, NULL, 0);
-    if (r < 0)
+    r = bf_wpack_new(&pack);
+    if (r)
         return r;
 
-    {
-        _cleanup_free_ struct bf_marsh *child = NULL;
-
-        r = bf_ctx_save(&child);
-        if (r < 0)
-            return r;
-
-        r = bf_marsh_add_child_obj(&marsh, child);
-        if (r < 0)
-            return r;
-    }
+    bf_wpack_open_object(pack, "ctx");
+    r = bf_ctx_save(pack);
+    if (r)
+        return r;
+    bf_wpack_close_object(pack);
 
+    /** @todo Front packing should be name-based to avoid relying on the
+     * enumeration order. */
+    bf_wpack_open_array(pack, "cache");
     for (int i = 0; i < _BF_FRONT_MAX; ++i) {
-        _cleanup_free_ struct bf_marsh *child = NULL;
-
-        if (!bf_opts_is_front_enabled(i))
-            continue;
-
-        r = bf_marsh_new(&child, NULL, 0);
-        if (r < 0)
-            return r;
-
-        r = bf_front_ops_get(i)->marsh(&child);
-        if (r < 0)
-            return r;
-
-        r = bf_marsh_add_child_obj(&marsh, child);
-        if (r < 0)
-            return r;
+        if (bf_opts_is_front_enabled(i)) {
+            bf_wpack_open_object(pack, NULL);
+            r = bf_front_ops_get(i)->pack(pack);
+            if (r < 0)
+                return r;
+            bf_wpack_close_object(pack);
+        } else {
+            bf_wpack_nil(pack);
+        }
     }
+    bf_wpack_close_array(pack);
 
-    r = bf_write_file(path, marsh, bf_marsh_size(marsh));
+    r = bf_wpack_get_data(pack, &data, &data_len);
+    if (r)
+        return r;
+
+    r = bf_write_file(path, data, data_len);
     if (r < 0)
         return r;
 
-    bf_dbg("saved marshalled context to %s", path);
+    bf_dbg("saved serialized context to %s", path);
 
     return 0;
 }
@@ -214,8 +202,6 @@ static int _bf_save(const char *path)
  *
  * Updated context is saved back to disk.
  *
- * @todo Should the runtime context be saved unconditionally?
- *
  * @return 0 on success, negative error code on failure.
  */
 static int _bf_init(int argc, char *argv[])
@@ -310,7 +296,7 @@ static int _bf_clean(void)
 /**
  * Process a request.
  *
- * The handler corresponding to @p request->front will be called (if any).
+ * The handler corresponding to @p bf_request_front(request) will be called (if any).
  * If the handler returns 0, @p response is expected to be filled, and ready
  * to be returned to the client.
  * If the handler returns a negative error code, @p response is filled by @ref
@@ -333,29 +319,31 @@ static int _bf_process_request(struct bf_request *request,
 
     bf_assert(request && response);
 
-    if (request->front < 0 || request->front >= _BF_FRONT_MAX) {
+    if (bf_request_front(request) < 0 ||
+        bf_request_front(request) >= _BF_FRONT_MAX) {
         bf_warn("received a request from front %d, unknown front, ignoring",
-                request->front);
+                bf_request_front(request));
         return bf_response_new_failure(response, -EINVAL);
     }
 
-    if (request->cmd < 0 || request->cmd >= _BF_REQ_CMD_MAX) {
+    if (bf_request_cmd(request) < 0 ||
+        bf_request_cmd(request) >= _BF_REQ_CMD_MAX) {
         bf_warn("received a request with command %d, unknown command, ignoring",
-                request->cmd);
+                bf_request_cmd(request));
         return bf_response_new_failure(response, -EINVAL);
     }
 
-    if (!bf_opts_is_front_enabled(request->front)) {
+    if (!bf_opts_is_front_enabled(bf_request_front(request))) {
         bf_warn("received a request from %s, but front is disabled, ignoring",
-                bf_front_to_str(request->front));
+                bf_front_to_str(bf_request_front(request)));
         return bf_response_new_failure(response, -ENOTSUP);
     }
 
     bf_info("processing request %s from %s",
-            bf_request_cmd_to_str(request->cmd),
-            bf_front_to_str(request->front));
+            bf_request_cmd_to_str(bf_request_cmd(request)),
+            bf_front_to_str(bf_request_front(request)));
 
-    ops = bf_front_ops_get(request->front);
+    ops = bf_front_ops_get(bf_request_front(request));
     r = ops->request_handler(request, response);
     if (r) {
         /* We failed to process the request, so we need to generate an
@@ -364,13 +352,14 @@ static int _bf_process_request(struct bf_request *request,
         r = bf_response_new_failure(response, r);
     }
 
-    if (!bf_opts_transient() && (request->cmd == BF_REQ_RULESET_FLUSH ||
-                                 request->cmd == BF_REQ_RULESET_SET ||
-                                 request->cmd == BF_REQ_CHAIN_SET ||
-                                 request->cmd == BF_REQ_CHAIN_LOAD ||
-                                 request->cmd == BF_REQ_CHAIN_ATTACH ||
-                                 request->cmd == BF_REQ_CHAIN_UPDATE ||
-                                 request->cmd == BF_REQ_CHAIN_FLUSH))
+    if (!bf_opts_transient() &&
+        (bf_request_cmd(request) == BF_REQ_RULESET_FLUSH ||
+         bf_request_cmd(request) == BF_REQ_RULESET_SET ||
+         bf_request_cmd(request) == BF_REQ_CHAIN_SET ||
+         bf_request_cmd(request) == BF_REQ_CHAIN_LOAD ||
+         bf_request_cmd(request) == BF_REQ_CHAIN_ATTACH ||
+         bf_request_cmd(request) == BF_REQ_CHAIN_UPDATE ||
+         bf_request_cmd(request) == BF_REQ_CHAIN_FLUSH))
         r = _bf_save(ctx_path);
 
     return r;
@@ -437,42 +426,45 @@ static int _bf_run(void)
                 continue;
             }
 
-            return bf_err_r(errno, "failed to accept connection");
+            bf_err_r(errno, "failed to accept connection, ignoring");
+            continue;
         }
 
         // NOLINTNEXTLINE: SOL_SOCKET and SO_PEERCRED can't be directly included
         r = getsockopt(client_fd, SOL_SOCKET, SO_PEERCRED, &peer_cred,
                        &peer_cred_len);
         if (r) {
-            bf_warn_r(
-                errno,
-                "failed to read the client's credentials, ignoring request");
+            bf_err_r(errno,
+                     "failed to read the client's credentials, ignoring");
             continue;
         }
 
         r = bf_ns_init(&ns, peer_cred.pid);
         if (r) {
-            bf_warn_r(
-                r, "failed to open the client's namespaces, ignoring request");
+            bf_err_r(r, "failed to open the client's namespaces, ignoring");
             continue;
         }
 
         r = bf_recv_request(client_fd, &request);
-        if (r < 0)
-            return bf_err_r(r, "failed to receive request");
+        if (r) {
+            bf_err_r(r, "failed to receive request, ignoring");
+            continue;
+        }
 
-        request->ns = &ns;
-        request->fd = client_fd;
+        bf_request_set_ns(request, &ns);
+        bf_request_set_fd(request, client_fd);
 
         r = _bf_process_request(request, &response);
         if (r) {
-            bf_err("failed to process request");
+            bf_err_r(r, "failed to process request, ignoring");
             continue;
         }
 
         r = bf_send_response(client_fd, response);
-        if (r < 0)
-            return bf_err_r(r, "failed to send response");
+        if (r) {
+            bf_err_r(r, "failed to send response, ignoring");
+            continue;
+        }
     }
 
     return 0;
