@@ -5,32 +5,35 @@
 
 #pragma once
 
-#include "core/front.h"
+#include <bpfilter/front.h>
+#include <bpfilter/pack.h>
 
 struct bf_request;
 struct bf_response;
-struct bf_marsh;
 
 /**
  * @struct bf_front_ops
  *
- * @todo Make bf_front_ops.request_handler take a const struct bf_request.
- *
- * @var bf_front_ops::setup
- *  Setup the front.
- * @var bf_front_ops::teardown
- *  Teardown the front.
- * @var bf_front_ops::request_handler
- *  Handle a request.
+ * @todo Front should not implement a callback if it's not needed. E.g.
+ * `BF_FRONT_CLI` defines empty `pack` and `unpack` callbacks.
  */
 struct bf_front_ops
 {
+    /// Initialize the front.
     int (*setup)(void);
+
+    /// Teardown the front and free resources.
     int (*teardown)(void);
-    int (*request_handler)(struct bf_request *request,
+
+    /// Handle an incoming request.
+    int (*request_handler)(const struct bf_request *request,
                            struct bf_response **response);
-    int (*marsh)(struct bf_marsh **marsh);
-    int (*unmarsh)(struct bf_marsh *marsh);
+
+    /// Serialize the front's data to restore it later.
+    int (*pack)(bf_wpack_t *pack);
+
+    /// Restore the front's data from serialized data.
+    int (*unpack)(bf_rpack_node_t node);
 };
 
 /**
