@@ -16,27 +16,28 @@
 #include <stdlib.h>
 #include <string.h>
 
-#include "bpfilter/cgen/cgen.h"
-#include "bpfilter/cgen/program.h"
-#include "bpfilter/ctx.h"
-#include "bpfilter/opts.h"
-#include "bpfilter/xlate/front.h"
-#include "bpfilter/xlate/ipt/dump.h"
-#include "bpfilter/xlate/ipt/helpers.h"
-#include "core/chain.h"
-#include "core/counter.h"
-#include "core/dump.h"
-#include "core/front.h"
-#include "core/helper.h"
-#include "core/hook.h"
-#include "core/list.h"
-#include "core/logger.h"
-#include "core/marsh.h"
-#include "core/matcher.h"
-#include "core/request.h"
-#include "core/response.h"
-#include "core/rule.h"
-#include "core/verdict.h"
+#include <bpfilter/chain.h>
+#include <bpfilter/counter.h>
+#include <bpfilter/dump.h>
+#include <bpfilter/front.h>
+#include <bpfilter/helper.h>
+#include <bpfilter/hook.h>
+#include <bpfilter/list.h>
+#include <bpfilter/logger.h>
+#include <bpfilter/matcher.h>
+#include <bpfilter/request.h>
+#include <bpfilter/response.h>
+#include <bpfilter/rule.h>
+#include <bpfilter/verdict.h>
+
+#include "bpfilter/runtime.h"
+#include "cgen/cgen.h"
+#include "cgen/program.h"
+#include "ctx.h"
+#include "opts.h"
+#include "xlate/front.h"
+#include "xlate/ipt/dump.h"
+#include "xlate/ipt/helpers.h"
 
 /**
  * @file ipt.c
@@ -90,6 +91,9 @@
  * by @c iptables .
  */
 
+#define BF_IPT_PRIO_0 1000
+#define BF_IPT_PRIO_1 1001
+
 /**
  * Get size of an ipt_replace structure.
  *
@@ -150,7 +154,6 @@ static int _bf_verdict_to_ipt_target(enum bf_verdict verdict,
 
     bf_assert(ipt_tgt);
 
-    bf_info("target for verdict %d", verdict);
     switch (verdict) {
     case BF_VERDICT_ACCEPT:
         std_tgt->verdict = -2;
@@ -201,30 +204,48 @@ static int _bf_ipt_entry_to_rule(const struct ipt_entry *entry,
 
     // Match on source IPv4 address
     if (entry->ip.src.s_addr || entry->ip.smsk.s_addr) {
-        struct bf_matcher_ip4_addr addr = {
-            .addr = entry->ip.src.s_addr,
-            .mask = entry->ip.smsk.s_addr,
-        };
+        if (entry->ip.smsk.s_addr == ~(uint32_t)0) {
+            r = bf_rule_add_matcher(
+                _rule, BF_MATCHER_IP4_SADDR,
+                entry->ip.invflags & IPT_INV_SRCIP ? BF_MATCHER_NE :
+                                                     BF_MATCHER_EQ,
+                &entry->ip.src.s_addr, sizeof(entry->ip.src.s_addr));
+        } else {
+            struct bf_ip4_lpm_key key = {
+                .data = entry->ip.src.s_addr,
+                .prefixlen = __builtin_ctz(entry->ip.smsk.s_addr),
+            };
+            r = bf_rule_add_matcher(_rule, BF_MATCHER_IP4_SNET,
+                                    entry->ip.invflags & IPT_INV_SRCIP ?
+                                        BF_MATCHER_NE :
+                                        BF_MATCHER_EQ,
+                                    &key, sizeof(key));
+        }
 
-        r = bf_rule_add_matcher(
-            _rule, BF_MATCHER_IP4_SADDR,
-            entry->ip.invflags & IPT_INV_SRCIP ? BF_MATCHER_NE : BF_MATCHER_EQ,
-            &addr, sizeof(addr));
         if (r)
             return r;
     }
 
     // Match on destination IPv4 address
     if (entry->ip.dst.s_addr || entry->ip.dmsk.s_addr) {
-        struct bf_matcher_ip4_addr addr = {
-            .addr = entry->ip.dst.s_addr,
-            .mask = entry->ip.dmsk.s_addr,
-        };
+        if (entry->ip.dmsk.s_addr == ~(uint32_t)0) {
+            r = bf_rule_add_matcher(
+                _rule, BF_MATCHER_IP4_DADDR,
+                entry->ip.invflags & IPT_INV_DSTIP ? BF_MATCHER_NE :
+                                                     BF_MATCHER_EQ,
+                &entry->ip.dst.s_addr, sizeof(entry->ip.dst.s_addr));
+        } else {
+            struct bf_ip4_lpm_key key = {
+                .data = entry->ip.dst.s_addr,
+                .prefixlen = __builtin_ctz(entry->ip.dmsk.s_addr),
+            };
+            r = bf_rule_add_matcher(_rule, BF_MATCHER_IP4_DNET,
+                                    entry->ip.invflags & IPT_INV_DSTIP ?
+                                        BF_MATCHER_NE :
+                                        BF_MATCHER_EQ,
+                                    &key, sizeof(key));
+        }
 
-        r = bf_rule_add_matcher(
-            _rule, BF_MATCHER_IP4_DADDR,
-            entry->ip.invflags & IPT_INV_DSTIP ? BF_MATCHER_NE : BF_MATCHER_EQ,
-            &addr, sizeof(addr));
         if (r)
             return r;
     }
@@ -267,34 +288,47 @@ static int _bf_ipt_entry_to_rule(const struct ipt_entry *entry,
 static int _bf_rule_to_ipt_entry(const struct bf_rule *rule,
                                  struct ipt_entry *entry)
 {
-    struct bf_matcher_ip4_addr *addr;
+    const struct bf_ip4_lpm_key *net;
 
     bf_assert(entry && rule);
 
     bf_list_foreach (&rule->matchers, matcher_node) {
         struct bf_matcher *matcher = bf_list_node_get_data(matcher_node);
 
-        switch (matcher->type) {
+        switch (bf_matcher_get_type(matcher)) {
         case BF_MATCHER_IP4_SADDR:
-            if (matcher->op == BF_MATCHER_NE)
+            if (bf_matcher_get_op(matcher) == BF_MATCHER_NE)
                 entry->ip.invflags |= IPT_INV_SRCIP;
-            addr = (void *)&matcher->payload;
-            entry->ip.src.s_addr = addr->addr;
-            entry->ip.smsk.s_addr = addr->mask;
+            entry->ip.src.s_addr = *(uint32_t *)bf_matcher_payload(matcher);
+            entry->ip.smsk.s_addr = ~(uint32_t)0;
+            break;
+        case BF_MATCHER_IP4_SNET:
+            if (bf_matcher_get_op(matcher) == BF_MATCHER_NE)
+                entry->ip.invflags |= IPT_INV_SRCIP;
+            net = bf_matcher_payload(matcher);
+            entry->ip.src.s_addr = net->data;
+            entry->ip.smsk.s_addr = (~(uint32_t)0) << (32 - net->prefixlen);
             break;
         case BF_MATCHER_IP4_DADDR:
-            if (matcher->op == BF_MATCHER_NE)
+            if (bf_matcher_get_op(matcher) == BF_MATCHER_NE)
                 entry->ip.invflags |= IPT_INV_DSTIP;
-            addr = (void *)&matcher->payload;
-            entry->ip.dst.s_addr = addr->addr;
-            entry->ip.dmsk.s_addr = addr->mask;
+            entry->ip.dst.s_addr = *(uint32_t *)bf_matcher_payload(matcher);
+            entry->ip.dmsk.s_addr = ~(uint32_t)0;
+            break;
+        case BF_MATCHER_IP4_DNET:
+            if (bf_matcher_get_op(matcher) == BF_MATCHER_NE)
+                entry->ip.invflags |= IPT_INV_DSTIP;
+            net = bf_matcher_payload(matcher);
+            entry->ip.dst.s_addr = net->data;
+            entry->ip.dmsk.s_addr = (~(uint32_t)0) << (32 - net->prefixlen);
             break;
         case BF_MATCHER_IP4_PROTO:
-            entry->ip.proto = *(uint8_t *)&matcher->payload;
+            entry->ip.proto = *(uint8_t *)bf_matcher_payload(matcher);
             break;
         default:
-            return bf_err_r(-ENOTSUP, "unsupported matcher %s for BF_FRONT_IPT",
-                            bf_matcher_type_to_str(matcher->type));
+            return bf_err_r(
+                -ENOTSUP, "unsupported matcher %s for BF_FRONT_IPT",
+                bf_matcher_type_to_str(bf_matcher_get_type(matcher)));
         }
     }
 
@@ -392,8 +426,8 @@ static int _bf_ipt_gen_get_ruleset(struct bf_ipt_gen_ruleset_entry *ruleset,
     /* iptables requires at least the INPUT, FORWARD, and OUTPUT chains. If
      * those chains are not defined, we created dummy ones just to fill the
      * ipt_replace structure. */
-    for (enum nf_inet_hooks hook = NF_INET_LOCAL_IN; hook <= NF_INET_LOCAL_OUT;
-         ++hook) {
+    for (enum bf_nf_inet_hooks hook = BF_NF_INET_LOCAL_IN;
+         hook <= BF_NF_INET_LOCAL_OUT; ++hook) {
         _free_bf_chain_ struct bf_chain *chain = NULL;
 
         if (ruleset[hook].cgen)
@@ -560,7 +594,7 @@ static int _bf_ipt_gen_ipt_replace(struct ipt_replace **replace,
  * @return 0 on success, negative error code on failure.
  */
 static int
-_bf_ipt_xlate_ruleset_set(struct ipt_replace *ipt,
+_bf_ipt_xlate_ruleset_set(const struct ipt_replace *ipt,
                           struct bf_chain *(*chains)[NF_INET_NUMHOOKS])
 {
     int r;
@@ -591,24 +625,21 @@ _bf_ipt_xlate_ruleset_set(struct ipt_replace *ipt,
 /**
  * Modify existing iptables rules.
  *
- * @todo If processing for any codegen fails, all codegens should be unloaded
- * and/or discarded.
- *
  * @param req The request sent to bpfilter. Can't be NULL.
  * @return 0 on success, negative error code on failure.
  */
 static int _bf_ipt_ruleset_set(const struct bf_request *req)
 {
-    struct ipt_replace *replace;
+    const struct ipt_replace *replace;
     struct bf_chain *chains[NF_INET_NUMHOOKS] = {};
-    bf_list _cur_cgens = bf_list_default(NULL, NULL);
+    _clean_bf_list_ bf_list _cur_cgens = bf_list_default(NULL, NULL);
     struct bf_cgen *cur_cgens[NF_INET_NUMHOOKS] = {};
     int r;
 
     bf_assert(req);
 
-    replace = (struct ipt_replace *)req->data;
-    if (bf_ipt_replace_size(replace) != req->data_len)
+    replace = bf_request_data(req);
+    if (bf_ipt_replace_size(replace) != bf_request_data_len(req))
         return -EINVAL;
 
     bf_ipt_dump_replace(replace, EMPTY_PREFIX);
@@ -623,7 +654,7 @@ static int _bf_ipt_ruleset_set(const struct bf_request *req)
 
     bf_list_foreach (&_cur_cgens, cgen_node) {
         struct bf_cgen *cgen = bf_list_node_get_data(cgen_node);
-        enum nf_inet_hooks hook = bf_hook_to_nf_hook(cgen->chain->hook);
+        enum bf_nf_inet_hooks hook = bf_hook_to_nf_hook(cgen->chain->hook);
 
         if (cur_cgens[hook])
             return bf_err_r(-EEXIST,
@@ -657,7 +688,13 @@ static int _bf_ipt_ruleset_set(const struct bf_request *req)
             if (r)
                 return r;
 
-            r = bf_cgen_attach(cgen, req->ns, &hookopts);
+            hookopts->family = PF_INET;
+            hookopts->priorities[0] = BF_IPT_PRIO_0;
+            hookopts->priorities[1] = BF_IPT_PRIO_1;
+            hookopts->used_opts |=
+                BF_FLAGS(BF_HOOKOPTS_FAMILY, BF_HOOKOPTS_PRIORITIES);
+
+            r = bf_cgen_attach(cgen, bf_request_ns(req), &hookopts);
             if (r) {
                 bf_err(
                     "failed to load a program for iptables hook %d, skipping",
@@ -700,25 +737,29 @@ static int _bf_ipt_ruleset_set(const struct bf_request *req)
  * @param len Length of the counters structure.
  * @return 0 on success, negative error code on failure.
  */
-static int _bf_ipt_set_counters_handler(struct xt_counters_info *counters,
+static int _bf_ipt_set_counters_handler(const struct xt_counters_info *counters,
                                         size_t len)
 {
-    bf_assert(counters);
-
+    UNUSED(counters);
     UNUSED(len);
 
     return 0;
 }
 
-int _bf_ipt_get_info_handler(struct bf_request *request,
+int _bf_ipt_get_info_handler(const struct bf_request *request,
                              struct bf_response **response)
 {
     _cleanup_free_ struct ipt_replace *replace = NULL;
-    struct ipt_getinfo *info = (struct ipt_getinfo *)request->data;
+    _cleanup_free_ struct ipt_getinfo *info = NULL;
     int r;
 
     bf_assert(request);
-    bf_assert(sizeof(*info) == request->data_len);
+    bf_assert(sizeof(*info) == bf_request_data_len(request));
+
+    info = bf_memdup(bf_request_data(request), bf_request_data_len(request));
+    if (!info) {
+        return -ENOMEM;
+    }
 
     if (!bf_streq(info->name, "filter")) {
         return bf_err_r(-EINVAL, "can't process IPT_SO_GET_INFO for table %s",
@@ -746,17 +787,19 @@ int _bf_ipt_get_info_handler(struct bf_request *request,
  * @param response
  * @return 0 on success, negative errno value on failure.
  */
-int _bf_ipt_get_entries_handler(struct bf_request *request,
+int _bf_ipt_get_entries_handler(const struct bf_request *request,
                                 struct bf_response **response)
 {
     _cleanup_free_ struct ipt_replace *replace = NULL;
-    struct ipt_get_entries *entries;
+    _cleanup_free_ struct ipt_get_entries *entries = NULL;
     int r;
 
     bf_assert(request);
     bf_assert(response);
 
-    entries = (struct ipt_get_entries *)request->data;
+    entries = bf_memdup(bf_request_data(request), bf_request_data_len(request));
+    if (!entries)
+        return -ENOMEM;
 
     if (!bf_streq(entries->name, "filter")) {
         return bf_err_r(-EINVAL, "can't process IPT_SO_GET_INFO for table %s",
@@ -790,43 +833,29 @@ static int _bf_ipt_teardown(void)
     return 0;
 }
 
-/**
- * @todo Wouldn't it be better to have a separate handler for each request type?
- *  In which case struct bf_front_ops would contain a handler for each request
- *  type, and the front would handle custom (BF_REQ_CUSTOM) requests itself.
- * @todo Document that request and responses are not const: they will be free
- *  by the daemon once the front is done with them. Hence, the front is free
- *  to modify the requests content.
- * @todo Check bf_assertions: a malformed request could cause the daemon to
- * crash.
- *
- * @param request
- * @param response
- * @return
- */
-static int _bf_ipt_request_handler(struct bf_request *request,
+static int _bf_ipt_request_handler(const struct bf_request *request,
                                    struct bf_response **response)
 {
     int r;
 
-    switch (request->cmd) {
+    switch (bf_request_cmd(request)) {
     case BF_REQ_RULESET_SET:
         r = _bf_ipt_ruleset_set(request);
         if (r < 0)
             return r;
 
-        return bf_response_new_success(response, request->data,
-                                       request->data_len);
+        return bf_response_new_success(response, bf_request_data(request),
+                                       bf_request_data_len(request));
     case BF_REQ_COUNTERS_SET:
-        r = _bf_ipt_set_counters_handler(
-            (struct xt_counters_info *)request->data, request->data_len);
+        r = _bf_ipt_set_counters_handler(bf_request_data(request),
+                                         bf_request_data_len(request));
         if (r < 0)
             return r;
 
-        return bf_response_new_success(response, request->data,
-                                       request->data_len);
+        return bf_response_new_success(response, bf_request_data(request),
+                                       bf_request_data_len(request));
     case BF_REQ_CUSTOM:
-        switch (request->ipt_cmd) {
+        switch (bf_request_ipt_cmd(request)) {
         case IPT_SO_GET_INFO:
             return _bf_ipt_get_info_handler(request, response);
         case IPT_SO_GET_ENTRIES:
@@ -834,26 +863,26 @@ static int _bf_ipt_request_handler(struct bf_request *request,
         default:
             return bf_warn_r(-ENOTSUP,
                              "unsupported custom ipt request type: %d",
-                             request->ipt_cmd);
+                             bf_request_ipt_cmd(request));
         };
     default:
         return bf_warn_r(-ENOTSUP, "unsupported ipt request type: %d",
-                         request->cmd);
+                         bf_request_ipt_cmd(request));
     };
 
     return 0;
 }
 
-static int _bf_ipt_marsh(struct bf_marsh **marsh)
+static int _bf_ipt_pack(bf_wpack_t *pack)
 {
-    UNUSED(marsh);
+    UNUSED(pack);
 
     return 0;
 }
 
-static int _bf_ipt_unmarsh(struct bf_marsh *marsh)
+static int _bf_ipt_unpack(bf_rpack_node_t node)
 {
-    UNUSED(marsh);
+    UNUSED(node);
 
     return 0;
 }
@@ -862,6 +891,6 @@ const struct bf_front_ops ipt_front = {
     .setup = _bf_ipt_setup,
     .teardown = _bf_ipt_teardown,
     .request_handler = _bf_ipt_request_handler,
-    .marsh = _bf_ipt_marsh,
-    .unmarsh = _bf_ipt_unmarsh,
+    .pack = _bf_ipt_pack,
+    .unpack = _bf_ipt_unpack,
 };
