@@ -5,11 +5,12 @@
 
 #include <errno.h>
 
-#include "bpfilter/xlate/front.h"
-#include "core/helper.h"
-#include "core/marsh.h"
-#include "core/request.h"
-#include "core/response.h"
+#include <bpfilter/helper.h>
+#include <bpfilter/pack.h>
+#include <bpfilter/request.h>
+#include <bpfilter/response.h>
+
+#include "xlate/front.h"
 
 static int _bf_nft_setup(void)
 {
@@ -21,21 +22,21 @@ static int _bf_nft_teardown(void)
     return 0;
 }
 
-static int _bf_nft_marsh(struct bf_marsh **marsh)
+static int _bf_nft_pack(bf_wpack_t *pack)
 {
-    UNUSED(marsh);
+    UNUSED(pack);
 
     return 0;
 }
 
-static int _bf_nft_unmarsh(struct bf_marsh *marsh)
+static int _bf_nft_unpack(bf_rpack_node_t node)
 {
-    UNUSED(marsh);
+    UNUSED(node);
 
     return 0;
 }
 
-static int _bf_nft_request_handler(struct bf_request *request,
+static int _bf_nft_request_handler(const struct bf_request *request,
                                    struct bf_response **response)
 {
     UNUSED(request);
@@ -49,6 +50,6 @@ const struct bf_front_ops nft_front = {
     .setup = _bf_nft_setup,
     .teardown = _bf_nft_teardown,
     .request_handler = _bf_nft_request_handler,
-    .marsh = _bf_nft_marsh,
-    .unmarsh = _bf_nft_unmarsh,
+    .pack = _bf_nft_pack,
+    .unpack = _bf_nft_unpack,
 };
