@@ -28,6 +28,7 @@ BFCLI=
 _BPFILTER= # bpfilter binary path
 BPFILTER= # bpfilter command to use in tests (includes the required options)
 SETUSERNS=
+RULESETS_DIR=
 
 # Colors
 BLUE='\033[0;34m'
@@ -55,11 +56,12 @@ log() {
 usage() {
     echo "Usage: $0 [OPTIONS]"
     echo "Options:"
-    echo "  --bfcli PATH      Path to bfcli executable"
-    echo "  --bpfilter PATH   Path to bpfilter executable"
-    echo "  --setuserns PATH  Path to the tool used to setup the user namespace"
-    echo "  --early-exit      Exit immediately on first test failure"
-    echo "  -h, --help        Display this help message and exit"
+    echo "  --bfcli PATH        Path to bfcli executable"
+    echo "  --bpfilter PATH     Path to bpfilter executable"
+    echo "  --setuserns PATH    Path to the tool used to setup the user namespace"
+    echo "  --rulesets-dir PATH Path to the directory containing the test rulesets"
+    echo "  --early-exit        Exit immediately on first test failure"
+    echo "  -h, --help          Display this help message and exit"
     exit 1
 }
 
@@ -90,6 +92,14 @@ while [[ $# -gt 0 ]]; do
             SETUSERNS=$(realpath $2)
             shift 2
             ;;
+        --rulesets-dir)
+            if [[ -z "$2" || "$2" == --* ]]; then
+                echo "Error: --rulesets-dir requires a path argument."
+                usage
+            fi
+            RULESETS_DIR=$(realpath $2)
+            shift 2
+            ;;
         --early-exit)
             EARLY_EXIT=1
             shift
@@ -182,6 +192,7 @@ setup() {
     log "${BLUE}    bfcli: ${BFCLI}"
     log "${BLUE}    bpfilter: ${_BPFILTER}"
     log "${BLUE}    setuserns: ${SETUSERNS}"
+    log "${BLUE}    rulesets-dir: ${RULESETS_DIR}"
 }
 
 cleanup() {
@@ -357,6 +368,24 @@ fi
 
 WITH_TIMEOUT="timeout --signal INT --preserve-status .5"
 
+expect_ruleset_ok() {
+    local ruleset="$1"
+
+    CHAIN="chain xdp BF_HOOK_XDP ACCEPT ${ruleset}"
+
+    expect_success "valid ruleset '${ruleset}'" \
+        ${FROM_NS} ${BFCLI} ruleset set --from-str \"${CHAIN}\"
+}
+
+expect_ruleset_nok() {
+    local ruleset="$1"
+
+    CHAIN="chain xdp BF_HOOK_XDP ACCEPT ${ruleset}"
+
+    expect_failure "invalid ruleset '${ruleset}'" \
+        ${FROM_NS} ${BFCLI} ruleset set --from-str \"${CHAIN}\"
+}
+
 expect_matcher_ok() {
     local matcher="$1"
 
@@ -399,6 +428,42 @@ suite_netns_to_host() {
 }
 with_daemon suite_netns_to_host
 
+suite_bcli_options_error() {
+    log "[SUITE] bfcli: options error"
+    expect_failure "ruleset set: --from-str and --from-file are incompatible" \
+        ${FROM_NS} ${BFCLI} ruleset set --from-str \"\" --from-file \"\"
+    expect_failure "ruleset set: --from-str or --from-file are required" \
+        ${FROM_NS} ${BFCLI} ruleset set
+
+    expect_failure "chain set: --from-str and --from-file are incompatible" \
+        ${FROM_NS} ${BFCLI} chain set --from-str \"\" --from-file \"\"
+    expect_failure "chain set: --from-str or --from-file are required" \
+        ${FROM_NS} ${BFCLI} chain set
+
+    expect_failure "chain get: --name is required" \
+        ${FROM_NS} ${BFCLI} chain get
+
+    expect_failure "chain logs: --name is required" \
+        ${FROM_NS} ${BFCLI} chain logs
+
+    expect_failure "chain load: --from-str and --from-file are incompatible" \
+        ${FROM_NS} ${BFCLI} chain load --from-str \"\" --from-file \"\"
+    expect_failure "chain load: --from-str or --from-file are required" \
+        ${FROM_NS} ${BFCLI} chain load
+
+    expect_failure "chain attach: --name is required" \
+        ${FROM_NS} ${BFCLI} chain attach
+
+    expect_failure "chain attach: --from-str and --from-file are incompatible" \
+        ${FROM_NS} ${BFCLI} chain attach --from-str \"\" --from-file \"\"
+    expect_failure "chain attach: --from-str or --from-file are required" \
+        ${FROM_NS} ${BFCLI} chain attach
+
+    expect_failure "chain flush: --name is required" \
+        ${FROM_NS} ${BFCLI} chain flush
+}
+with_daemon suite_bcli_options_error
+
 suite_host_to_netns() {
     log "[SUITE] netns: host -> netns"
     expect_failure "can't attach chain to host iface from netns" \
@@ -446,28 +511,6 @@ suite_icmp_XDP() {
 }
 with_daemon suite_icmp_XDP
 
-suite_ip4() {
-    log "[SUITE] ip4: parse matchers"
-    expect_success "ip4.snet in" \
-	    ${FROM_NS} ${BFCLI} ruleset set --from-str \"chain xdp BF_HOOK_XDP ACCEPT rule ip4.snet in \{192.168.1.14/24,10.211.0.0/16\} DROP\"
-    expect_success "ip4.dnet in" \
-	    ${FROM_NS} ${BFCLI} ruleset set --from-str \"chain xdp BF_HOOK_XDP ACCEPT rule ip4.dnet in \{192.168.1.14/24,10.211.0.0/16\} DROP\"
-    expect_success "flushing the ruleset" \
-        ${FROM_NS} ${BFCLI} ruleset flush
-}
-with_daemon suite_ip4
-
-suite_ip6() {
-    log "[SUITE] ip6: parse matchers"
-    expect_success "ip6.snet in" \
-	    ${FROM_NS} ${BFCLI} ruleset set --from-str \"chain xdp BF_HOOK_XDP ACCEPT rule ip6.snet in \{fdb2:2c26:f4e4:0:21c:42ff:fe09:1a95/64,fe80::21c:42ff:fe09:1a95/64\} log link DROP\"
-    expect_success "ip6.dnet in" \
-	    ${FROM_NS} ${BFCLI} ruleset set --from-str \"chain xdp BF_HOOK_XDP ACCEPT rule ip6.dnet in \{fdb2:2c26:f4e4:0:21c:42ff:fe09:1a95/64,fe80::21c:42ff:fe09:1a95/64\} log internet DROP\"
-    expect_success "flushing the ruleset" \
-        ${FROM_NS} ${BFCLI} ruleset flush
-}
-with_daemon suite_ip6
-
 suite_icmpv6_chain_set() {
     log "[SUITE] icmpv6: chain set"
     expect_success "can parse icmpv6 type and code by TC chain" \
@@ -720,6 +763,57 @@ suite_daemon_restore_non_attached() {
 }
 without_daemon suite_daemon_restore_non_attached
 
+suite_daemon_pin_updated_chain() {
+    log "[SUITE] daemon: check if updated chain is pinned"
+
+    start_daemon
+        expect_success "create a chain and attach it" \
+            ${FROM_NS} ${BFCLI} chain set --from-str \"chain test_chain BF_HOOK_XDP{ifindex=${NS_IFINDEX}} ACCEPT\"
+        expect_success "can ping the netns iface from the host" \
+            ${FROM_NS} ping -c 1 -W 0.25 ${NS_IP_ADDR}
+        expect_success "after set the first chain, bpfilter only has a single BPF program" \
+            ${FROM_NS} ${BFCLI} ruleset get \| grep "^chain" \| awk 'END{exit\ NR!=1}'
+        expect_success "after set the first chain, bpftool only has a single bf_prog BPF program" \
+            bpftool prog \| grep \"name bf_prog\" \| awk 'END{exit\ NR!=1}'
+
+        expect_success "create a chain and attach it" \
+            ${FROM_NS} ${BFCLI} chain update --from-str \"chain test_chain BF_HOOK_XDP{ifindex=${NS_IFINDEX}} ACCEPT rule meta.l4_proto eq icmp DROP\"
+        expect_failure "can't ping the netns iface from the host" \
+            ping -c 1 -W 0.25 ${NS_IP_ADDR}
+        expect_success "after updating the chain, bpfilter only has a single BPF program" \
+            ${FROM_NS} ${BFCLI} ruleset get \| grep "^chain" \| awk 'END{exit\ NR!=1}'
+        expect_success "after updating the chain, bpftool only has a single bf_prog BPF program" \
+            bpftool prog \| grep \"name bf_prog\" \| awk 'END{exit\ NR!=1}'
+    stop_daemon
+
+    start_daemon
+        expect_success "after restarting the daemon, bpfilter only has a single BPF program" \
+            ${FROM_NS} ${BFCLI} ruleset get \| grep "^chain" \| awk 'END{exit\ NR!=1}'
+        expect_success "after restarting the daemon, bpftool only has a single bf_prog BPF program" \
+            bpftool prog \| grep \"name bf_prog\" \| awk 'END{exit\ NR!=1}'
+    stop_daemon
+}
+without_daemon suite_daemon_pin_updated_chain
+
+suite_rule_meta_order() {
+    log "[SUITE] cli: allow out of order log, counter"
+    expect_failure "duplicate keyword counter" \
+        ${FROM_NS} ${BFCLI} chain set --from-str \"chain order BF_HOOK_XDP ACCEPT rule ip4.proto icmp counter counter DROP\"
+    expect_failure "duplicate keyword log" \
+        ${FROM_NS} ${BFCLI} chain set --from-str \"chain order BF_HOOK_XDP ACCEPT rule ip4.proto icmp counter log link log link DROP\"
+    expect_success "none of log, counter" \
+        ${FROM_NS} ${BFCLI} chain set --from-str \"chain order BF_HOOK_XDP ACCEPT rule ip4.proto icmp DROP\"
+    expect_success "only log" \
+        ${FROM_NS} ${BFCLI} chain set --from-str \"chain order BF_HOOK_XDP ACCEPT rule ip4.proto icmp log link DROP\"
+    expect_success "only counter" \
+        ${FROM_NS} ${BFCLI} chain set --from-str \"chain order BF_HOOK_XDP ACCEPT rule ip4.proto icmp counter DROP\"
+    expect_success "use log then counter" \
+        ${FROM_NS} ${BFCLI} chain set --from-str \"chain order BF_HOOK_XDP ACCEPT rule ip4.proto icmp log link counter DROP\"
+    expect_success "use counter then log" \
+        ${FROM_NS} ${BFCLI} chain set --from-str \"chain order BF_HOOK_XDP ACCEPT rule ip4.proto icmp counter log link DROP\"
+}
+with_daemon suite_rule_meta_order
+
 suite_log() {
     log "[SUITE] cli: log"
     expect_failure "invalid log action" \
@@ -737,6 +831,139 @@ suite_log() {
 }
 with_daemon suite_log
 
+suite_mark() {
+    log "[SUITE] cli: mark"
+    expect_failure "incompatible chain (XDP)" \
+        ${FROM_NS} ${BFCLI} chain set --from-str \"chain xdp_mark BF_HOOK_XDP ACCEPT rule ip4.proto icmp mark 0x16 DROP\"
+    expect_failure "incompatible chain (Netfilter)" \
+        ${FROM_NS} ${BFCLI} chain set --from-str \"chain xdp_mark BF_HOOK_NF_LOCAL_IN ACCEPT rule ip4.proto icmp mark 0x16 DROP\"
+    expect_failure "missing mark value" \
+        ${FROM_NS} ${BFCLI} chain set --from-str \"chain xdp_mark BF_HOOK_TC_INGRESS ACCEPT rule ip4.proto icmp mark DROP\"
+    expect_failure "invalid mark value (can't parse)" \
+        ${FROM_NS} ${BFCLI} chain set --from-str \"chain xdp_mark BF_HOOK_TC_INGRESS ACCEPT rule ip4.proto icmp mark 0x14aw DROP\"
+    expect_failure "invalid mark value (negative value)" \
+        ${FROM_NS} ${BFCLI} chain set --from-str \"chain xdp_mark BF_HOOK_TC_INGRESS ACCEPT rule ip4.proto icmp mark -3 DROP\"
+    expect_failure "invalid mark value (value too big)" \
+        ${FROM_NS} ${BFCLI} chain set --from-str \"chain xdp_mark BF_HOOK_TC_INGRESS ACCEPT rule ip4.proto icmp mark 0xffffffffff DROP\"
+
+    expect_success "valid decimal value" \
+        ${FROM_NS} ${BFCLI} chain set --from-str \"chain xdp_mark BF_HOOK_TC_INGRESS ACCEPT rule ip4.proto icmp mark 14 DROP\"
+    expect_success "valid hexadecimal value" \
+        ${FROM_NS} ${BFCLI} chain set --from-str \"chain xdp_mark BF_HOOK_TC_INGRESS ACCEPT rule ip4.proto icmp mark 0x14 DROP\"
+}
+with_daemon suite_mark
+
+suite_matcher_set() {
+    log "[SUITE] matcher: set"
+    expect_matcher_ok "(ip4.saddr, icmp.code) in {192.168.1.1,41; 192.168.1.1,42}"
+    expect_matcher_ok "(ip4.saddr, icmp.code) in {192.168.1.1 ,41; 192.168.1.1,42}"
+    expect_matcher_ok "(ip4.saddr, icmp.code) in {192.168.1.1, 41; 192.168.1.1,42}"
+    expect_matcher_ok "(ip4.saddr, icmp.code) in {192.168.1.1,41;192.168.1.1,42}"
+    expect_matcher_ok "(ip4.saddr, icmp.code) in {
+        192.168.1.1 , 41;
+        192.168.1.1,42
+    }"
+    expect_matcher_ok "(ip6.saddr) in {
+        ::1;
+        ::2
+    }"
+    expect_matcher_ok "(ip6.snet) in {
+        ::1/100;
+        ::2/89
+    }"
+    expect_matcher_ok "(ip4.saddr, icmp.code) in {
+        192.168.1.1 , 41 ;
+        192.168.1.1 , 42 ;
+    }"
+
+    # Testing all the matchers
+    expect_matcher_ok "(ip4.saddr, ip4.daddr) in {
+        192.168.1.1, 192.168.1.2;
+        192.168.1.3, 192.168.1.4
+    }"
+    expect_matcher_ok "(ip4.proto, ip6.nexthdr) in {6, 40; 40, 6}"
+    expect_matcher_ok "(ip6.saddr, ip6.daddr) in {
+        ::1, ::2;
+        ::3, ::4
+    }"
+    expect_matcher_ok "(icmp.code, icmp.type) in {
+        3, echo-reply;
+        2, echo-request
+    }"
+    expect_matcher_ok "(icmpv6.code, icmpv6.type) in {
+        3, echo-reply;
+        2, echo-request
+    }"
+    expect_matcher_ok "(icmpv6.code, icmpv6.type   ) in {
+        3, echo-reply;
+        2, echo-request
+    }"
+    expect_matcher_ok "(icmpv6.code   , icmpv6.type   ) in {
+        3, echo-reply;
+        2, echo-request
+    }"
+    expect_matcher_ok "(  icmpv6.code,      icmpv6.type   ) in {
+        3, echo-reply;
+        2, echo-request
+    }"
+
+    expect_matcher_nok "(ip4.snet, ip4.dnet) in {
+        192.168.1.1/24, 192.167.1.1/24;
+        10.211.55.2/24, 192.168.1.1/24
+    }"
+    expect_matcher_nok "(ip6.snet, ip6.dnet) in {
+        ::1/32, ::2/64;
+        ::3/96, ::4/128
+    }"
+    expect_matcher_nok "(ip4.saddr, ) in {192.168.1.1,41; 192.168.1.1,42}"
+    expect_matcher_nok "(ip4.saddr, icmp.code) in {192.168.1.1,41 192.168.1.1,42}"
+    expect_matcher_nok "(ip4.saddr, icmp.code) in {192.168.1.141; 192.168.1.1,42}"
+    expect_matcher_nok "(ip4.saddr, icmp.code) in {192.168.1.1}"
+    expect_matcher_nok "(ip4.saddr;icmp.code) in {192.168.1.1,41; 192.168.1.1,42}"
+    expect_matcher_nok "(ip4.saddr, icmp.code) in {}"
+    expect_matcher_nok "(ip4.saddr, icmp.code) in {192.168.1.,41; 192.168.1.1,42}"
+    expect_matcher_nok "(ip4.saddr, icmp.code) in {192.168.1.1,cafe; 192.168.1.1,42}"
+    expect_matcher_nok "(ip4.saddr, icmp.code) in {192.168.1.1,41,192.168.1.1,42}"
+}
+with_daemon suite_matcher_set
+
+suite_matcher_named_set() {
+    log "[SUITE] matcher: named set"
+    expect_ruleset_ok "
+        set myset (ip4.saddr) in {
+            192.168.1.1;
+            192.168.1.2
+        }
+        rule
+            (ip4.saddr) in myset
+            counter
+            ACCEPT
+    "
+    expect_ruleset_ok "
+        set myset (ip4.saddr, ip4.proto) in {
+            192.168.1.1, tcp;
+            192.168.1.2, udp
+        }
+        rule
+            (ip4.saddr, ip4.proto) in myset
+            counter
+            ACCEPT
+    "
+
+    expect_ruleset_nok "set myset (ip4.saddr) eq { 192.168.1.1 }"
+    expect_ruleset_nok "set myset (ip4.saddr, meta.ifindex) in { 192.168.1.1 }"
+    expect_ruleset_nok "set myset (ip4.saddr, ip4.proto) in { 192.168.1.1 }"
+    expect_ruleset_nok "
+        set myset (ip4.saddr) in { 192.168.1.1 }
+        rule (ip4.daddr) in myset
+    "
+    expect_ruleset_nok "
+        set myset (ip4.saddr) in { 192.168.1.1 }
+        rule (ip4.daddr) in my_set
+    "
+}
+with_daemon suite_matcher_named_set
+
 suite_matcher_meta() {
     log "[SUITE] matcher: meta.iface"
     expect_matcher_ok "meta.iface eq lo"
@@ -888,6 +1115,26 @@ suite_matcher_meta() {
     expect_matcher_nok "meta.probability eq 1000%"
     expect_matcher_nok "meta.probability eq 15.5%"
     expect_matcher_nok "meta.probability eq teapot"
+
+    # Do not use expect_matcher_ok/nok as meta.mark is not compatible with XDP
+    log "[SUITE] matcher: meta.mark eq"
+    expect_success "valid matcher 'meta.mark eq 0'" \
+        ${FROM_NS} ${BFCLI} ruleset set --from-str \"chain xdp BF_HOOK_TC_INGRESS ACCEPT rule meta.mark eq 0 counter DROP\"
+    expect_success "valid matcher 'meta.mark eq 15'" \
+        ${FROM_NS} ${BFCLI} ruleset set --from-str \"chain xdp BF_HOOK_TC_INGRESS ACCEPT rule meta.mark eq 15 counter DROP\"
+    expect_success "valid matcher 'meta.mark eq 0x00'" \
+        ${FROM_NS} ${BFCLI} ruleset set --from-str \"chain xdp BF_HOOK_TC_INGRESS ACCEPT rule meta.mark eq 0x00 counter DROP\"
+    expect_success "valid matcher 'meta.mark eq 0xffffffff'" \
+        ${FROM_NS} ${BFCLI} ruleset set --from-str \"chain xdp BF_HOOK_TC_INGRESS ACCEPT rule meta.mark eq 0xffffffff counter DROP\"
+
+    expect_failure "invalid matcher 'meta.mark eq -1'" \
+        ${FROM_NS} ${BFCLI} ruleset set --from-str \"chain xdp BF_HOOK_TC_INGRESS ACCEPT rule meta.mark eq -1 counter DROP\"
+    expect_failure "invalid matcher 'meta.mark eq 0xffffffffff'" \
+        ${FROM_NS} ${BFCLI} ruleset set --from-str \"chain xdp BF_HOOK_TC_INGRESS ACCEPT rule meta.mark eq 0xffffffffff counter DROP\"
+    expect_failure "invalid matcher 'meta.mark eq 1qw'" \
+        ${FROM_NS} ${BFCLI} ruleset set --from-str \"chain xdp BF_HOOK_TC_INGRESS ACCEPT rule meta.mark eq 1qw counter DROP\"
+    expect_failure "invalid matcher 'meta.mark eq qw'" \
+        ${FROM_NS} ${BFCLI} ruleset set --from-str \"chain xdp BF_HOOK_TC_INGRESS ACCEPT rule meta.mark eq qw counter DROP\"
 }
 with_daemon suite_matcher_meta
 
@@ -1637,6 +1884,14 @@ suite_matcher_icmpv6() {
 }
 with_daemon suite_matcher_icmpv6
 
+suite_generate_for_each_hook() {
+    log "[SUITE] cli: generate a chain for each hook"
+    for file in "$RULESETS_DIR"/*.bf; do
+        expect_success "generate chain for $(basename $file .bf)" \
+            ${FROM_NS} ${BFCLI} chain set --from-file ${file}
+    done
+}
+with_daemon suite_generate_for_each_hook
 
 ################################################################################
 #
