@@ -3,9 +3,11 @@
  * Copyright (c) 2023 Meta Platforms, Inc. and affiliates.
  */
 
-#include "bpfilter/cgen/runtime.h"
-#include "core/chain.h"
-#include "core/logger.h"
+#include "cgen/runtime.h"
+#include "bpfilter/chain.h"
+#include "bpfilter/logger.h"
+#include "bpfilter/matcher.h"
+#include "bpfilter/runtime.h"
 #include "e2e.h"
 #include "harness/filters.h"
 #include "harness/test.h"
@@ -67,6 +69,65 @@ Test(counters, update_partially_disabled)
                               bft_counter_p(1, 1, BFT_NO_BYTES));
 }
 
+Test(counters, packet_size)
+{
+    // Counters should be properly updated, even though some rules have counters
+    // disabled
+    _free_bf_chain_ struct bf_chain *chain = bf_test_chain_get(
+        BF_HOOK_XDP, BF_VERDICT_ACCEPT, NULL,
+        (struct bf_rule *[]) {
+            // Do not match
+            bf_rule_get(0, true, BF_VERDICT_ACCEPT,
+                        (struct bf_matcher *[]) {
+                            bf_matcher_get(BF_MATCHER_IP4_SADDR, BF_MATCHER_EQ,
+                                           (uint8_t[]) {
+                                                127, 2, 10, 10
+                                           },
+                                           4),
+                            NULL,
+                        }),
+            NULL,
+        });
+
+    bft_e2e_test_with_counter(chain, BF_VERDICT_ACCEPT, pkt_local_ip4,
+                              bft_counter_p(0, 1, pkt_local_ip4[0].pkt_len));
+}
+
+Test(mark, set_mark_then_read)
+{
+    _free_bf_rule_ struct bf_rule *rule_mark = 
+        bf_rule_get(0, true, BF_VERDICT_CONTINUE, (struct bf_matcher *[]) {
+            bf_matcher_get(BF_MATCHER_IP4_SADDR, BF_MATCHER_EQ,
+                (uint8_t[]) {
+                    127, 2, 10, 10
+                },
+                4),
+            NULL,
+        });
+    bf_rule_mark_set(rule_mark, 0xff);
+
+    _free_bf_rule_ struct bf_rule *rule_match = 
+        bf_rule_get(0, true, BF_VERDICT_ACCEPT, (struct bf_matcher *[]) {
+            bf_matcher_get(BF_MATCHER_META_MARK, BF_MATCHER_EQ,
+                (uint32_t[]) {
+                    0xff
+                },
+                4),
+            NULL,
+        });
+
+    _free_bf_chain_ struct bf_chain *chain = bf_test_chain_get(
+        BF_HOOK_XDP, BF_VERDICT_ACCEPT, NULL,
+        (struct bf_rule *[]) {
+            TAKE_PTR(rule_mark),
+            TAKE_PTR(rule_match),
+            NULL,
+         });
+
+    bft_e2e_test_with_counter(chain, BF_VERDICT_ACCEPT, pkt_local_ip4,
+                              bft_counter_p(1, 1, pkt_local_ip4[0].pkt_len));
+}
+
 Test(meta, l4_proto)
 {
     _free_bf_chain_ struct bf_chain *match_eq = bf_test_chain_get(
@@ -167,6 +228,39 @@ Test(meta, l4_proto)
     bft_e2e_test(nomatch_ne, BF_VERDICT_ACCEPT, pkt_remote_ip6_eh_tcp);
 }
 
+Test(meta, probability)
+{
+    _free_bf_chain_ struct bf_chain *always_drop = bf_test_chain_get(
+        BF_HOOK_XDP, BF_VERDICT_ACCEPT, NULL,
+        (struct bf_rule *[]) {
+            bf_rule_get(
+                0, false, BF_VERDICT_DROP,
+                (struct bf_matcher *[]) {
+                    bf_matcher_get(BF_MATCHER_META_PROBABILITY, BF_MATCHER_EQ,
+                                   (uint8_t[]) { 100 },
+                                   1),
+                    NULL,
+                }),
+            NULL,
+        });
+    bft_e2e_test(always_drop, BF_VERDICT_DROP, pkt_local_ip6_hop);
+
+    _free_bf_chain_ struct bf_chain *never_drop = bf_test_chain_get(
+        BF_HOOK_XDP, BF_VERDICT_ACCEPT, NULL,
+        (struct bf_rule *[]) {
+            bf_rule_get(
+                0, false, BF_VERDICT_DROP,
+                (struct bf_matcher *[]) {
+                    bf_matcher_get(BF_MATCHER_META_PROBABILITY, BF_MATCHER_EQ,
+                                   (uint8_t[]) { 0 },
+                                   1),
+                    NULL,
+                }),
+            NULL,
+        });
+    bft_e2e_test(never_drop, BF_VERDICT_ACCEPT, pkt_local_ip6_hop);
+}
+
 Test(ip4, proto)
 {
     _free_bf_chain_ struct bf_chain *match_eq = bf_test_chain_get(
@@ -488,12 +582,11 @@ Test(ip4, snet)
                 BF_VERDICT_DROP,
                 (struct bf_matcher *[]) {
                     bf_matcher_get(BF_MATCHER_IP4_SNET, BF_MATCHER_EQ,
-                        (uint8_t[]) {
-                            // IP address
-                            0x7f, 0x02, 0x00, 0x0c,
-                            0xff, 0xff, 0x00, 0x00
-                        },
-                        8
+                        (struct bf_ip4_lpm_key[]){{
+                            .prefixlen = 16,
+                            .data = 0x0c00027f,
+                        }},
+                        sizeof(struct bf_ip4_lpm_key)
                     ),
                     NULL,
                 }
@@ -513,12 +606,11 @@ Test(ip4, snet)
                 BF_VERDICT_DROP,
                 (struct bf_matcher *[]) {
                     bf_matcher_get(BF_MATCHER_IP4_SNET, BF_MATCHER_EQ,
-                        (uint8_t[]) {
-                            // IP address
-                            0x7f, 0x02, 0x00, 0x0c,
-                            0xff, 0xff, 0xff, 0x00
-                        },
-                        8
+                        (struct bf_ip4_lpm_key[]){{
+                            .prefixlen = 24,
+                            .data = 0x0c00027f,
+                        }},
+                        sizeof(struct bf_ip4_lpm_key)
                     ),
                     NULL,
                 }
@@ -538,12 +630,11 @@ Test(ip4, snet)
                 BF_VERDICT_DROP,
                 (struct bf_matcher *[]) {
                     bf_matcher_get(BF_MATCHER_IP4_SNET, BF_MATCHER_NE,
-                        (uint8_t[]) {
-                            // IP address
-                            0x7f, 0x02, 0x00, 0x0c,
-                            0xff, 0xff, 0xff, 0x00
-                        },
-                        8
+                        (struct bf_ip4_lpm_key[]){{
+                            .prefixlen = 24,
+                            .data = 0x0c00027f,
+                        }},
+                        sizeof(struct bf_ip4_lpm_key)
                     ),
                     NULL,
                 }
@@ -563,12 +654,11 @@ Test(ip4, snet)
                 BF_VERDICT_DROP,
                 (struct bf_matcher *[]) {
                     bf_matcher_get(BF_MATCHER_IP4_SNET, BF_MATCHER_NE,
-                        (uint8_t[]) {
-                            // IP address
-                            0x7f, 0x02, 0x00, 0x0c,
-                            0xff, 0xff, 0x00, 0x00
-                        },
-                        8
+                        (struct bf_ip4_lpm_key[]){{
+                            .prefixlen = 16,
+                            .data = 0x0c00027f,
+                        }},
+                        sizeof(struct bf_ip4_lpm_key)
                     ),
                     NULL,
                 }
@@ -596,12 +686,11 @@ Test(ip4, dnet)
                 BF_VERDICT_DROP,
                 (struct bf_matcher *[]) {
                     bf_matcher_get(BF_MATCHER_IP4_DNET, BF_MATCHER_EQ,
-                        (uint8_t[]) {
-                            // IP address
-                            0x7f, 0x02, 0x0b, 0x0c,
-                            0xff, 0xff, 0x00, 0x00,
-                        },
-                        8
+                        (struct bf_ip4_lpm_key[]) {{
+                            .prefixlen = 16,
+                            .data = 0x0c0b027f,
+                        }},
+                        sizeof(struct bf_ip4_lpm_key)
                     ),
                     NULL,
                 }
@@ -621,12 +710,11 @@ Test(ip4, dnet)
                 BF_VERDICT_DROP,
                 (struct bf_matcher *[]) {
                     bf_matcher_get(BF_MATCHER_IP4_DNET, BF_MATCHER_EQ,
-                        (uint8_t[]) {
-                            // IP address
-                            0x7f, 0x02, 0x0b, 0x0c,
-                            0xff, 0xff, 0xff, 0x00,
-                        },
-                        8
+                        (struct bf_ip4_lpm_key[]) {{
+                            .prefixlen = 24,
+                            .data = 0x0c0b027f,
+                        }},
+                        sizeof(struct bf_ip4_lpm_key)
                     ),
                     NULL,
                 }
@@ -646,12 +734,11 @@ Test(ip4, dnet)
                 BF_VERDICT_DROP,
                 (struct bf_matcher *[]) {
                     bf_matcher_get(BF_MATCHER_IP4_DNET, BF_MATCHER_NE,
-                        (uint8_t[]) {
-                            // IP address
-                            0x7f, 0x02, 0x0b, 0x0c,
-                            0xff, 0xff, 0xff, 0x00,
-                        },
-                        8
+                        (struct bf_ip4_lpm_key[]) {{
+                            .prefixlen = 24,
+                            .data = 0x0c0b027f,
+                        }},
+                        sizeof(struct bf_ip4_lpm_key)
                     ),
                     NULL,
                 }
@@ -671,12 +758,11 @@ Test(ip4, dnet)
                 BF_VERDICT_DROP,
                 (struct bf_matcher *[]) {
                     bf_matcher_get(BF_MATCHER_IP4_DNET, BF_MATCHER_NE,
-                        (uint8_t[]) {
-                            // IP address
-                            0x7f, 0x02, 0x0b, 0x0c,
-                            0xff, 0xff, 0x00, 0x00,
-                        },
-                        8
+                        (struct bf_ip4_lpm_key[]) {{
+                            .prefixlen = 16,
+                            .data = 0x0c0b027f,
+                        }},
+                        sizeof(struct bf_ip4_lpm_key)
                     ),
                     NULL,
                 }
@@ -729,7 +815,10 @@ Test(ip4, snet_in)
         BF_VERDICT_ACCEPT,
         (struct bf_set *[]) {
             bft_set_get(
-                BF_SET_IP4_SUBNET,
+                (enum bf_matcher_type[]) {
+                    BF_MATCHER_IP4_SNET
+                },
+                1,
                 (struct bf_ip4_lpm_key []){
                     (struct bf_ip4_lpm_key) {
                         .prefixlen = 24,
@@ -746,7 +835,7 @@ Test(ip4, snet_in)
                 false,
                 BF_VERDICT_DROP,
                 (struct bf_matcher *[]) {
-                    bf_matcher_get(BF_MATCHER_IP4_SNET, BF_MATCHER_IN,
+                    bf_matcher_get(BF_MATCHER_SET, BF_MATCHER_IN,
                         (uint32_t[]) {0}, 4
                     ),
                     NULL,
@@ -763,7 +852,10 @@ Test(ip4, snet_in)
         BF_VERDICT_ACCEPT,
         (struct bf_set *[]) {
             bft_set_get(
-                BF_SET_IP4_SUBNET,
+                (enum bf_matcher_type[]) {
+                    BF_MATCHER_IP4_SNET
+                },
+                1,
                 (struct bf_ip4_lpm_key []){
                     (struct bf_ip4_lpm_key) {
                         .prefixlen = 24,
@@ -781,7 +873,7 @@ Test(ip4, snet_in)
                 false,
                 BF_VERDICT_DROP,
                 (struct bf_matcher *[]) {
-                    bf_matcher_get(BF_MATCHER_IP4_SNET, BF_MATCHER_IN,
+                    bf_matcher_get(BF_MATCHER_SET, BF_MATCHER_IN,
                         (uint32_t[]) {0}, 4
                     ),
                     NULL,
@@ -801,7 +893,10 @@ Test(ip4, dnet_in)
         BF_VERDICT_ACCEPT,
         (struct bf_set *[]) {
             bft_set_get(
-                BF_SET_IP4_SUBNET,
+                (enum bf_matcher_type[]) {
+                    BF_MATCHER_IP4_DNET
+                },
+                1,
                 (struct bf_ip4_lpm_key []){
                     (struct bf_ip4_lpm_key) {
                         .prefixlen = 24,
@@ -818,7 +913,7 @@ Test(ip4, dnet_in)
                 false,
                 BF_VERDICT_DROP,
                 (struct bf_matcher *[]) {
-                    bf_matcher_get(BF_MATCHER_IP4_DNET, BF_MATCHER_IN,
+                    bf_matcher_get(BF_MATCHER_SET, BF_MATCHER_IN,
                         (uint32_t[]) {0}, 4
                     ),
                     NULL,
@@ -835,7 +930,10 @@ Test(ip4, dnet_in)
         BF_VERDICT_ACCEPT,
         (struct bf_set *[]) {
             bft_set_get(
-                BF_SET_IP4_SUBNET,
+                (enum bf_matcher_type[]) {
+                    BF_MATCHER_IP4_DNET
+                },
+                1,
                 (struct bf_ip4_lpm_key []){
                     (struct bf_ip4_lpm_key) {
                         .prefixlen = 24,
@@ -853,7 +951,7 @@ Test(ip4, dnet_in)
                 false,
                 BF_VERDICT_DROP,
                 (struct bf_matcher *[]) {
-                    bf_matcher_get(BF_MATCHER_IP4_DNET, BF_MATCHER_IN,
+                    bf_matcher_get(BF_MATCHER_SET, BF_MATCHER_IN,
                         (uint32_t[]) {0}, 4
                     ),
                     NULL,
@@ -1077,15 +1175,14 @@ Test(ip6, snet)
             bf_rule_get(0, false, BF_VERDICT_DROP,
                         (struct bf_matcher *[]) {
                             bf_matcher_get(BF_MATCHER_IP6_SNET, BF_MATCHER_EQ,
-                                (uint8_t[]) {
-                                    /* IPv6 address */
-                                    0x54, 0x2c, 0x1a, 0x31, 0xf9, 0x64, 0x94, 0x6c,
-                                    0x5a, 0x24, 0xe7, 0x1e, 0x4d, 0x26, 0xb8, 0x7e,
-                                    /* Mask */
-                                    0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
-                                    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
-                                },
-                                32
+                                (struct bf_ip6_lpm_key[]) {{
+                                    .prefixlen = 64,
+                                    .data = {
+                                        0x54, 0x2c, 0x1a, 0x31, 0xf9, 0x64, 0x94, 0x6c,
+                                        0x5a, 0x24, 0xe7, 0x1e, 0x4d, 0x26, 0xb8, 0x7e,
+                                    }
+                                }},
+                                sizeof(struct bf_ip6_lpm_key)
                             ),
                             NULL,
                         }),
@@ -1098,15 +1195,14 @@ Test(ip6, snet)
             bf_rule_get(0, false, BF_VERDICT_DROP,
                         (struct bf_matcher *[]) {
                             bf_matcher_get(BF_MATCHER_IP6_SNET, BF_MATCHER_EQ,
-                                (uint8_t[]) {
-                                    /* IPv6 address */
-                                    0x54, 0x2c, 0x1a, 0x31, 0xf9, 0x64, 0x94, 0x6c,
-                                    0x5b /* Changed */, 0x24, 0xe7, 0x1e, 0x4d, 0x26, 0xb8, 0x7e,
-                                    /* Mask */
-                                    0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
-                                    0xff, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
-                                },
-                                32
+                                (struct bf_ip6_lpm_key[]) {{
+                                    .prefixlen = 72,
+                                    .data = {
+                                        0x54, 0x2c, 0x1a, 0x31, 0xf9, 0x64, 0x94, 0x6c,
+                                        0x5b /* Changed */, 0x24, 0xe7, 0x1e, 0x4d, 0x26, 0xb8, 0x7e,
+                                    }
+                                }},
+                                sizeof(struct bf_ip6_lpm_key)
                             ),
                             NULL,
                         }),
@@ -1119,15 +1215,14 @@ Test(ip6, snet)
             bf_rule_get(0, false, BF_VERDICT_DROP,
                         (struct bf_matcher *[]) {
                             bf_matcher_get(BF_MATCHER_IP6_SNET, BF_MATCHER_NE,
-                                (uint8_t[]) {
-                                    /* IPv6 address */
-                                    0x54, 0x2c, 0x1a, 0x31, 0xf9, 0x64, 0x94, 0x6c,
-                                    0x5b /* Changed */, 0x24, 0xe7, 0x1e, 0x4d, 0x26, 0xb8, 0x7e,
-                                    /* Mask */
-                                    0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
-                                    0xff, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
-                                },
-                                32
+                                (struct bf_ip6_lpm_key[]) {{
+                                    .prefixlen = 72,
+                                    .data = {
+                                        0x54, 0x2c, 0x1a, 0x31, 0xf9, 0x64, 0x94, 0x6c,
+                                        0x5b /* Changed */, 0x24, 0xe7, 0x1e, 0x4d, 0x26, 0xb8, 0x7e,
+                                    }
+                                }},
+                                sizeof(struct bf_ip6_lpm_key)
                             ),
                             NULL,
                         }),
@@ -1140,15 +1235,14 @@ Test(ip6, snet)
             bf_rule_get(0, false, BF_VERDICT_DROP,
                         (struct bf_matcher *[]) {
                             bf_matcher_get(BF_MATCHER_IP6_SNET, BF_MATCHER_NE,
-                                (uint8_t[]) {
-                                    /* IPv6 address */
-                                    0x54, 0x2c, 0x1a, 0x31, 0xf9, 0x64, 0x94, 0x6c,
-                                    0x5a, 0x24, 0xe7, 0x1e, 0x4d, 0x26, 0xb8, 0x7e,
-                                    /* Mask */
-                                    0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
-                                    0xff, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
-                                },
-                                32
+                                (struct bf_ip6_lpm_key[]) {{
+                                    .prefixlen = 72,
+                                    .data = {
+                                        0x54, 0x2c, 0x1a, 0x31, 0xf9, 0x64, 0x94, 0x6c,
+                                        0x5a, 0x24, 0xe7, 0x1e, 0x4d, 0x26, 0xb8, 0x7e,
+                                    }
+                                }},
+                                sizeof(struct bf_ip6_lpm_key)
                             ),
                             NULL,
                         }),
@@ -1171,15 +1265,14 @@ Test(ip6, dnet)
             bf_rule_get(0, false, BF_VERDICT_DROP,
                         (struct bf_matcher *[]) {
                             bf_matcher_get(BF_MATCHER_IP6_DNET, BF_MATCHER_EQ,
-                                (uint8_t[]) {
-                                    /* IPv6 address */
-                                    0x52, 0x32, 0x18, 0x5a, 0x52, 0xf9, 0x0a, 0xb4,
-                                    0x80, 0x25, 0x79, 0x74, 0x22, 0x99, 0xeb, 0x04,
-                                    /* Mask */
-                                    0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
-                                    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
-                                },
-                                32
+                                (struct bf_ip6_lpm_key[]) {{
+                                    .prefixlen = 64,
+                                    .data = {
+                                        0x52, 0x32, 0x18, 0x5a, 0x52, 0xf9, 0x0a, 0xb4,
+                                        0x80, 0x25, 0x79, 0x74, 0x22, 0x99, 0xeb, 0x04,
+                                    }
+                                }},
+                                sizeof(struct bf_ip6_lpm_key)
                             ),
                             NULL,
                         }),
@@ -1192,15 +1285,14 @@ Test(ip6, dnet)
             bf_rule_get(0, false, BF_VERDICT_DROP,
                         (struct bf_matcher *[]) {
                             bf_matcher_get(BF_MATCHER_IP6_DNET, BF_MATCHER_EQ,
-                                (uint8_t[]) {
-                                    /* IPv6 address */
-                                    0x52, 0x32, 0x18, 0x5a, 0x52, 0xf9, 0x0a, 0xb4,
-                                    0x81 /* Changed */, 0x25, 0x79, 0x74, 0x22, 0x99, 0xeb, 0x04,
-                                    /* Mask */
-                                    0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
-                                    0xff, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
-                                },
-                                32
+                                (struct bf_ip6_lpm_key[]) {{
+                                    .prefixlen = 72,
+                                    .data = {
+                                        0x52, 0x32, 0x18, 0x5a, 0x52, 0xf9, 0x0a, 0xb4,
+                                        0x81 /* Changed */, 0x25, 0x79, 0x74, 0x22, 0x99, 0xeb, 0x04,
+                                    }
+                                }},
+                                sizeof(struct bf_ip6_lpm_key)
                             ),
                             NULL,
                         }),
@@ -1213,15 +1305,14 @@ Test(ip6, dnet)
             bf_rule_get(0, false, BF_VERDICT_DROP,
                         (struct bf_matcher *[]) {
                             bf_matcher_get(BF_MATCHER_IP6_DNET, BF_MATCHER_NE,
-                                (uint8_t[]) {
-                                    /* IPv6 address */
-                                    0x52, 0x32, 0x18, 0x5a, 0x52, 0xf9, 0x0a, 0xb4,
-                                    0x81 /* Changed */, 0x25, 0x79, 0x74, 0x22, 0x99, 0xeb, 0x04,
-                                    /* Mask */
-                                    0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
-                                    0xff, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
-                                },
-                                32
+                                (struct bf_ip6_lpm_key[]) {{
+                                    .prefixlen = 72,
+                                    .data = {
+                                        0x52, 0x32, 0x18, 0x5a, 0x52, 0xf9, 0x0a, 0xb4,
+                                        0x81 /* Changed */, 0x25, 0x79, 0x74, 0x22, 0x99, 0xeb, 0x04,
+                                    }
+                                }},
+                                sizeof(struct bf_ip6_lpm_key)
                             ),
                             NULL,
                         }),
@@ -1234,15 +1325,14 @@ Test(ip6, dnet)
             bf_rule_get(0, false, BF_VERDICT_DROP,
                         (struct bf_matcher *[]) {
                             bf_matcher_get(BF_MATCHER_IP6_DNET, BF_MATCHER_NE,
-                                (uint8_t[]) {
-                                    /* IPv6 address */
-                                    0x52, 0x32, 0x18, 0x5a, 0x52, 0xf9, 0x0a, 0xb4,
-                                    0x80, 0x25, 0x79, 0x74, 0x22, 0x99, 0xeb, 0x04,
-                                    /* Mask */
-                                    0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
-                                    0xff, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
-                                },
-                                32
+                                (struct bf_ip6_lpm_key[]) {{
+                                    .prefixlen = 72,
+                                    .data = {
+                                        0x52, 0x32, 0x18, 0x5a, 0x52, 0xf9, 0x0a, 0xb4,
+                                        0x80, 0x25, 0x79, 0x74, 0x22, 0x99, 0xeb, 0x04,
+                                    }
+                                }},
+                                sizeof(struct bf_ip6_lpm_key)
                             ),
                             NULL,
                         }),
@@ -1262,7 +1352,10 @@ Test(ip6, snet_in)
         BF_VERDICT_ACCEPT,
         (struct bf_set *[]) {
             bft_set_get(
-                BF_SET_IP6_SUBNET,
+                (enum bf_matcher_type[]) {
+                    BF_MATCHER_IP6_SNET
+                },
+                1,
                 (struct bf_ip6_lpm_key []){
                     (struct bf_ip6_lpm_key) {
                         .prefixlen = 64,
@@ -1282,7 +1375,7 @@ Test(ip6, snet_in)
                 false,
                 BF_VERDICT_DROP,
                 (struct bf_matcher *[]) {
-                    bf_matcher_get(BF_MATCHER_IP6_SNET, BF_MATCHER_IN,
+                    bf_matcher_get(BF_MATCHER_SET, BF_MATCHER_IN,
                         (uint32_t[]) {0}, 4
                     ),
                     NULL,
@@ -1299,7 +1392,10 @@ Test(ip6, snet_in)
         BF_VERDICT_ACCEPT,
         (struct bf_set *[]) {
             bft_set_get(
-                BF_SET_IP6_SUBNET,
+                (enum bf_matcher_type[]) {
+                    BF_MATCHER_IP6_SNET
+                },
+                1,
                 (struct bf_ip6_lpm_key []){
                     (struct bf_ip6_lpm_key) {
                         .prefixlen = 64,
@@ -1319,7 +1415,7 @@ Test(ip6, snet_in)
                 false,
                 BF_VERDICT_DROP,
                 (struct bf_matcher *[]) {
-                    bf_matcher_get(BF_MATCHER_IP6_SNET, BF_MATCHER_IN,
+                    bf_matcher_get(BF_MATCHER_SET, BF_MATCHER_IN,
                         (uint32_t[]) {0}, 4
                     ),
                     NULL,
@@ -1339,7 +1435,10 @@ Test(ip6, dnet_in)
         BF_VERDICT_ACCEPT,
         (struct bf_set *[]) {
             bft_set_get(
-                BF_SET_IP6_SUBNET,
+                (enum bf_matcher_type[]) {
+                    BF_MATCHER_IP6_DNET
+                },
+                1,
                 (struct bf_ip6_lpm_key []){
                     (struct bf_ip6_lpm_key) {
                         .prefixlen = 64,
@@ -1359,7 +1458,7 @@ Test(ip6, dnet_in)
                 false,
                 BF_VERDICT_DROP,
                 (struct bf_matcher *[]) {
-                    bf_matcher_get(BF_MATCHER_IP6_DNET, BF_MATCHER_IN,
+                    bf_matcher_get(BF_MATCHER_SET, BF_MATCHER_IN,
                         (uint32_t[]) {0}, 4
                     ),
                     NULL,
@@ -1376,13 +1475,16 @@ Test(ip6, dnet_in)
         BF_VERDICT_ACCEPT,
         (struct bf_set *[]) {
             bft_set_get(
-                BF_SET_IP6_SUBNET,
+                (enum bf_matcher_type[]) {
+                    BF_MATCHER_IP6_DNET
+                },
+                1,
                 (struct bf_ip6_lpm_key []){
                     (struct bf_ip6_lpm_key) {
                         .prefixlen = 64,
                         .data = {
                             0x52, 0x32, 0x18, 0x5a, 0x52, 0xf9, 0x0a, 0xb4,
-                            0x80, 0x25, 0x79, 0x74, 0x22, 0x99, 0xeb, 0x04
+                            0x80, 0x25, 0x79, 0x74, /* Changed */ 0x23, 0x99, 0xeb, 0x04
                         },
                     },
                 },
@@ -1396,7 +1498,7 @@ Test(ip6, dnet_in)
                 false,
                 BF_VERDICT_DROP,
                 (struct bf_matcher *[]) {
-                    bf_matcher_get(BF_MATCHER_IP6_DNET, BF_MATCHER_IN,
+                    bf_matcher_get(BF_MATCHER_SET, BF_MATCHER_IN,
                         (uint32_t[]) {0}, 4
                     ),
                     NULL,
@@ -1409,188 +1511,6 @@ Test(ip6, dnet_in)
     bft_e2e_test(in, BF_VERDICT_DROP, pkt_remote_ip6_tcp);
 }
 
-struct bf_set *make_ip6port_set(size_t nelems, uint8_t *matching_elem)
-{
-    _free_bf_set_ struct bf_set *set = NULL;
-    int r;
-
-    r = bf_set_new(&set, BF_SET_SRCIP6PORT);
-    if (r < 0) {
-        bf_err_r(r, "failed to create a new set");
-        return NULL;
-    }
-
-    if (matching_elem) {
-        r = bf_set_add_elem(set, matching_elem);
-        if (r < 0) {
-            bf_err_r(r, "failed to add matching element to set");
-            return NULL;
-        }
-    }
-
-    for (size_t i = 0; i < nelems; i++) {
-        uint8_t elem[18] = {};
-
-        for (int j = 0; j < (int)ARRAY_SIZE(elem); j++)
-            elem[j] = rand() % 256;
-
-        r = bf_set_add_elem(set, elem);
-        if (r < 0) {
-            bf_err_r(r, "failed to add key to set");
-            return NULL;
-        }
-    }
-
-    return TAKE_PTR(set);
-}
-
-Test(ip6, port_200kset_match)
-{
-    _free_bf_chain_ struct bf_chain *chain = bf_test_chain_get(
-        BF_HOOK_XDP,
-        BF_VERDICT_ACCEPT,
-        (struct bf_set *[]) {
-            make_ip6port_set(2000, (uint8_t[]){0x54, 0x2c, 0x1a, 0x31, 0xf9, 0x64, 0x94, 0x6c, 0x5a, 0x24, 0xe7, 0x1e, 0x4d, 0x26, 0xb8, 0x7e, 0x7a, 0x69}),
-            NULL,
-        },
-        (struct bf_rule *[]) {
-            bf_rule_get(
-                0,
-                false,
-                BF_VERDICT_DROP,
-                (struct bf_matcher *[]) {
-                    bf_matcher_get(BF_MATCHER_SET_SRCIP6PORT, BF_MATCHER_IN,
-                        (uint32_t[]) {0}, 4
-                    ),
-                    NULL,
-                }
-            ),
-            NULL,
-        }
-    );
-
-    bft_e2e_test(chain, BF_VERDICT_DROP, pkt_remote_ip6_tcp);
-}
-
-Test(ip6, port_200kset_nomatch)
-{
-    _free_bf_chain_ struct bf_chain *chain = bf_test_chain_get(
-        BF_HOOK_XDP,
-        BF_VERDICT_ACCEPT,
-        (struct bf_set *[]) {
-            make_ip6port_set(2000, NULL),
-            NULL,
-        },
-        (struct bf_rule *[]) {
-            bf_rule_get(
-                0,
-                false,
-                BF_VERDICT_DROP,
-                (struct bf_matcher *[]) {
-                    bf_matcher_get(BF_MATCHER_SET_SRCIP6PORT, BF_MATCHER_IN,
-                        (uint32_t[]) {0}, 4
-                    ),
-                    NULL,
-                }
-            ),
-            NULL,
-        }
-    );
-
-    bft_e2e_test(chain, BF_VERDICT_ACCEPT, pkt_remote_ip6_tcp);
-}
-
-struct bf_set *make_ip6_set(size_t nelems, uint8_t *matching_elem)
-{
-    _free_bf_set_ struct bf_set *set = NULL;
-    int r;
-
-    r = bf_set_new(&set, BF_SET_SRCIP6);
-    if (r < 0) {
-        bf_err_r(r, "failed to create a new set");
-        return NULL;
-    }
-
-    if (matching_elem) {
-        r = bf_set_add_elem(set, matching_elem);
-        if (r < 0) {
-            bf_err_r(r, "failed to add matching element to set");
-            return NULL;
-        }
-    }
-
-    for (size_t i = 0; i < nelems; i++) {
-        uint8_t elem[16] = {};
-
-        for (int j = 0; j < (int)ARRAY_SIZE(elem); j++)
-            elem[j] = rand() % 256;
-
-        r = bf_set_add_elem(set, elem);
-        if (r < 0) {
-            bf_err_r(r, "failed to add key to set");
-            return NULL;
-        }
-    }
-
-    return TAKE_PTR(set);
-}
-
-Test(ip6, addrport_200kset_match)
-{
-    _free_bf_chain_ struct bf_chain *chain = bf_test_chain_get(
-        BF_HOOK_XDP,
-        BF_VERDICT_ACCEPT,
-        (struct bf_set *[]) {
-            make_ip6_set(2000, (uint8_t[]){0x54, 0x2c, 0x1a, 0x31, 0xf9, 0x64, 0x94, 0x6c, 0x5a, 0x24, 0xe7, 0x1e, 0x4d, 0x26, 0xb8, 0x7e}),
-            NULL,
-        },
-        (struct bf_rule *[]) {
-            bf_rule_get(
-                0,
-                false,
-                BF_VERDICT_DROP,
-                (struct bf_matcher *[]) {
-                    bf_matcher_get(BF_MATCHER_SET_SRCIP6PORT, BF_MATCHER_IN,
-                        (uint32_t[]) {0}, 4
-                    ),
-                    NULL,
-                }
-            ),
-            NULL,
-        }
-    );
-
-    bft_e2e_test(chain, BF_VERDICT_DROP, pkt_remote_ip6_tcp);
-}
-
-Test(ip6, addrport_200kset_nomatch)
-{
-    _free_bf_chain_ struct bf_chain *chain = bf_test_chain_get(
-        BF_HOOK_XDP,
-        BF_VERDICT_ACCEPT,
-        (struct bf_set *[]) {
-            make_ip6_set(2000, NULL),
-            NULL,
-        },
-        (struct bf_rule *[]) {
-            bf_rule_get(
-                0,
-                false,
-                BF_VERDICT_DROP,
-                (struct bf_matcher *[]) {
-                    bf_matcher_get(BF_MATCHER_SET_SRCIP6PORT, BF_MATCHER_IN,
-                        (uint32_t[]) {0}, 4
-                    ),
-                    NULL,
-                }
-            ),
-            NULL,
-        }
-    );
-
-    bft_e2e_test(chain, BF_VERDICT_ACCEPT, pkt_remote_ip6_tcp);
-}
-
 Test(tcp, dport_range)
 {
     _free_bf_chain_ struct bf_chain *in_range = bf_test_chain_get(
@@ -2229,7 +2149,7 @@ Test(ipv6, extension_headers)
                 BF_VERDICT_DROP,
                 (struct bf_matcher *[]) {
                     bf_matcher_get(BF_MATCHER_TCP_DPORT, BF_MATCHER_EQ,
-                        (uint8_t[]) { 0xb7, 0x7a },
+                        (uint8_t[]) { 0x7a, 0xb7 },
                         2
                     ),
                     NULL,
