@@ -8,12 +8,12 @@
 #include <stddef.h>
 #include <stdint.h>
 
-#include "core/chain.h"
-#include "core/hook.h"
-#include "core/matcher.h"
-#include "core/rule.h"
-#include "core/set.h"
-#include "core/verdict.h"
+#include "bpfilter/chain.h"
+#include "bpfilter/hook.h"
+#include "bpfilter/matcher.h"
+#include "bpfilter/rule.h"
+#include "bpfilter/set.h"
+#include "bpfilter/verdict.h"
 
 /**
  * @file filters.h
@@ -87,27 +87,25 @@
 struct bf_hookopts *bft_hookopts_get(const char *raw_opt, ...);
 
 /**
- * Create a new set.
+ * @brief Create a new test set.
  *
  * @code {.c}
  *  bft_set_get(
- *      BF_SET_IP4,
- *      (struct my_custom_struct []) {
- *          { 0x01, 0x02, 0x03, 0x04 },
- *      },
- *      1
+ *      (enum bf_matcher_type []){{ BF_MATCHER_IP4_PROTO }}, 1
+ *      (uint8_t[]) { IPPROTO_TCP, IPPROTO_UDP, }, 2
  *  );
  * @endcode
  *
  * The caller owns the set and is responsible for freeing it.
  *
- * @param type Set type. Defines the key size.
- * @param data Array of elements to fill the set with. The elements are
- *        expected to a size defined by their @p type .
+ * @param key Array of `bf_matcher_type` defining the key format.
+ * @param n_comps Number of matcher types in the key.
+ * @param data Array of elements to fill the set with.
  * @param n_elems Number of elements in @p data .
  * @return A valid @ref bf_set on success, or NULL on failure.
  */
-struct bf_set *bft_set_get(enum bf_set_type type, void *data, size_t n_elems);
+struct bf_set *bft_set_get(enum bf_matcher_type *key, size_t n_comps,
+                           void *data, size_t n_elems);
 
 /**
  * Create a new matcher.
@@ -145,3 +143,21 @@ struct bf_rule *bf_rule_get(uint8_t log, bool counters, enum bf_verdict verdict,
 struct bf_chain *bf_test_chain_get(enum bf_hook hook, enum bf_verdict policy,
                                    struct bf_set **sets,
                                    struct bf_rule **rules);
+
+struct bft_list_dummy_data
+{
+    size_t id;
+    size_t len;
+    uint8_t padding[];
+};
+
+typedef bool (*bft_list_eq_cb)(const void *lhs, const void *rhs);
+
+int bft_list_dummy_data_new_from_pack(struct bft_list_dummy_data **data,
+                                      bf_rpack_node_t node);
+int bft_list_dummy_data_pack(struct bft_list_dummy_data *data,
+                             bf_wpack_t *pack);
+bool bft_list_dummy_data_compare(const struct bft_list_dummy_data *lhs,
+                                 const struct bft_list_dummy_data *rhs);
+bf_list *bft_list_get(size_t n_elems, size_t elem_size);
+bool bft_list_eq(const bf_list *lhs, const bf_list *rhs, bft_list_eq_cb cb);
