@@ -3,7 +3,7 @@
  * Copyright (c) 2023 Meta Platforms, Inc. and affiliates.
  */
 
-#include "bpfilter/cgen/prog/map.c"
+#include "cgen/prog/map.c"
 
 #include "fake.h"
 #include "harness/test.h"
@@ -11,11 +11,11 @@
 
 Test(map, create_delete_assert)
 {
-    expect_assert_failure(bf_map_new(NULL, "012345", BF_MAP_TYPE_SET, BF_MAP_BPF_TYPE_ARRAY, 1, 1, 1));
-    expect_assert_failure(bf_map_new(NOT_NULL, NULL, BF_MAP_TYPE_SET, BF_MAP_BPF_TYPE_ARRAY, 1, 1, 1));
-    expect_assert_failure(bf_map_new(NOT_NULL, "", BF_MAP_TYPE_SET, BF_MAP_BPF_TYPE_ARRAY, 1, 1, 1));
-    expect_assert_failure(bf_map_new(NOT_NULL, "012345", BF_MAP_TYPE_SET, BF_MAP_BPF_TYPE_ARRAY, 1, 1, 0));
+    expect_assert_failure(bf_map_new(NULL, "012345", BF_MAP_TYPE_LOG, 1, 1, 1));
+    expect_assert_failure(bf_map_new(NOT_NULL, NULL, BF_MAP_TYPE_LOG, 1, 1, 1));
     expect_assert_failure(bf_map_free(NULL));
+    assert_error(bf_map_new(NOT_NULL, "", BF_MAP_TYPE_LOG, 1, 1, 1));
+    assert_error(bf_map_new(NOT_NULL, "012345", BF_MAP_TYPE_LOG, 1, 1, 0));
 }
 
 Test(map, create_delete)
@@ -23,13 +23,13 @@ Test(map, create_delete)
     // Rely on the cleanup attribute
     _free_bf_map_ struct bf_map *map0 = NULL;
 
-    assert_success(bf_map_new(&map0, "012345", BF_MAP_TYPE_SET, BF_MAP_BPF_TYPE_ARRAY, 1, 1, 1));
+    assert_success(bf_map_new(&map0, "012345", BF_MAP_TYPE_LOG, 1, 1, 1));
     assert_non_null(map0);
 
     // Use the cleanup attribute, but free manually
     _free_bf_map_ struct bf_map *map1 = NULL;
 
-    assert_success(bf_map_new(&map1, "012345", BF_MAP_TYPE_SET, BF_MAP_BPF_TYPE_ARRAY, 1, 1, 1));
+    assert_success(bf_map_new(&map1, "012345", BF_MAP_TYPE_LOG, 1, 1, 1));
     assert_non_null(map1);
 
     bf_map_free(&map1);
@@ -38,7 +38,7 @@ Test(map, create_delete)
     // Free manually
     struct bf_map *map2;
 
-    assert_success(bf_map_new(&map2, "012345", BF_MAP_TYPE_SET, BF_MAP_BPF_TYPE_ARRAY, 1, 1, 1));
+    assert_success(bf_map_new(&map2, "012345", BF_MAP_TYPE_LOG, 1, 1, 1));
     assert_non_null(map2);
 
     bf_map_free(&map2);
@@ -46,25 +46,27 @@ Test(map, create_delete)
     bf_map_free(&map2);
 }
 
-Test(map, marsh_unmarsh_assert)
-{
-    expect_assert_failure(bf_map_new_from_marsh(NULL, 0, NOT_NULL));
-    expect_assert_failure(bf_map_new_from_marsh(NOT_NULL, 0, NULL));
-    expect_assert_failure(bf_map_marsh(NULL, NOT_NULL));
-    expect_assert_failure(bf_map_marsh(NOT_NULL, NULL));
-}
-
-Test(map, marsh_unmarsh)
+Test(map, pack_unpack)
 {
     _free_bf_map_ struct bf_map *map0 = NULL;
     _free_bf_map_ struct bf_map *map1 = NULL;
-    _free_bf_marsh_ struct bf_marsh *marsh = NULL;
+    _free_bf_wpack_ bf_wpack_t *wpack = NULL;
+    _free_bf_rpack_ bf_rpack_t *rpack = NULL;
+    const void *data;
+    size_t data_len;
     _clean_bf_test_mock_ bf_test_mock _ = bf_test_mock_get(bf_bpf_obj_get, 0);
 
-    assert_success(bf_map_new(&map0, "012345", BF_MAP_TYPE_SET, BF_MAP_BPF_TYPE_ARRAY, 1, 2, 3));
+    expect_assert_failure(bf_map_pack(NULL, NOT_NULL));
+    expect_assert_failure(bf_map_pack(NOT_NULL, NULL));
 
-    assert_success(bf_map_marsh(map0, &marsh));
-    assert_success(bf_map_new_from_marsh(&map1, 0, marsh));
+    assert_success(bf_map_new(&map0, "012345", BF_MAP_TYPE_LOG, 1, 2, 3));
+
+    assert_success(bf_wpack_new(&wpack));
+    assert_success(bf_map_pack(map0, wpack));
+    assert_success(bf_wpack_get_data(wpack, &data, &data_len));
+
+    assert_success(bf_rpack_new(&rpack, data, data_len));
+    assert_success(bf_map_new_from_pack(&map1, 0, bf_rpack_root(rpack)));
 
     // Ensure we won't try to close a garbage FD
     map1->fd = -1;
@@ -87,16 +89,10 @@ Test(map, dump)
 {
     _free_bf_map_ struct bf_map *map = NULL;
 
-    assert_success(bf_map_new(&map, "012345", BF_MAP_TYPE_SET, BF_MAP_BPF_TYPE_ARRAY, 1, 1, 1));
+    assert_success(bf_map_new(&map, "012345", BF_MAP_TYPE_LOG, 1, 1, 1));
     bf_map_dump(map, EMPTY_PREFIX);
 }
 
-Test(map, bpf_map_type_to_kernel_type_assert)
-{
-    expect_assert_failure(_bf_map_bpf_type_to_kernel_type(-1));
-    expect_assert_failure(_bf_map_bpf_type_to_kernel_type(_BF_MAP_BPF_TYPE_MAX));
-}
-
 Test(map, btf_create_delete_assert)
 {
     expect_assert_failure(_bf_btf_new(NULL));
@@ -133,19 +129,19 @@ Test(map, btf_create_delete)
 
 Test(map, map_create_assert)
 {
-    expect_assert_failure(bf_map_create(NULL, 0));
+    expect_assert_failure(bf_map_create(NULL));
 }
 
 Test(map, map_create)
 {
     _free_bf_map_ struct bf_map *map = NULL;
-    _clean_bf_test_mock_ bf_test_mock _0 = bf_test_mock_get(bf_bpf, 16);
+    _clean_bf_test_mock_ bf_test_mock _0 = bf_test_mock_get(bf_bpf_map_create, 16);
     _clean_bf_test_mock_ bf_test_mock _1 = bf_test_mock_get(bf_ctx_token, -1);
 
-    assert_success(bf_map_new(&map, "suffix", BF_MAP_TYPE_SET, BF_MAP_BPF_TYPE_ARRAY, 1, 1, BF_MAP_N_ELEMS_UNKNOWN));
-    assert_error(bf_map_create(map, 0));
+    assert_success(bf_map_new(&map, "suffix", BF_MAP_TYPE_LOG, 1, 1, BF_MAP_N_ELEMS_UNKNOWN));
+    assert_error(bf_map_create(map));
     assert_success(bf_map_set_n_elems(map, 1));
-    assert_success(bf_map_create(map, 0));
+    assert_success(bf_map_create(map));
     assert_error(bf_map_set_n_elems(map, 1));
 
     // So bf_map_free() doesn't try to close a random FD value
@@ -155,11 +151,11 @@ Test(map, map_create)
 Test(map, map_create_failure)
 {
     _free_bf_map_ struct bf_map *map = NULL;
-    _clean_bf_test_mock_ bf_test_mock _0 = bf_test_mock_get(bf_bpf, -1);
+    _clean_bf_test_mock_ bf_test_mock _0 = bf_test_mock_get(bf_bpf_map_create, -1);
     _clean_bf_test_mock_ bf_test_mock _1 = bf_test_mock_get(bf_ctx_token, -1);
 
-    assert_success(bf_map_new(&map, "suffix", BF_MAP_TYPE_SET, BF_MAP_BPF_TYPE_ARRAY, 1, 1, 1));
-    assert_error(bf_map_create(map, 0));
+    assert_success(bf_map_new(&map, "suffix", BF_MAP_TYPE_LOG, 1, 1, 1));
+    assert_error(bf_map_create(map));
 }
 
 Test(map, map_destroy_assert)
@@ -173,26 +169,3 @@ Test(map, map_set_elem_assert)
     expect_assert_failure(bf_map_set_elem(NOT_NULL, NULL, NOT_NULL));
     expect_assert_failure(bf_map_set_elem(NOT_NULL, NOT_NULL, NULL));
 }
-
-Test(map, bpf_map_type_to_from_assert)
-{
-    expect_assert_failure(bf_map_bpf_type_to_str(-1));
-    expect_assert_failure(bf_map_bpf_type_to_str(_BF_MAP_BPF_TYPE_MAX));
-    expect_assert_failure(bf_map_bpf_type_from_str(NULL, NOT_NULL));
-    expect_assert_failure(bf_map_bpf_type_from_str(NOT_NULL, NULL));
-}
-
-Test(map, bpf_map_type_to_from)
-{
-    enum bf_map_bpf_type type;
-
-    for (size_t i = 0; i < _BF_MAP_BPF_TYPE_MAX; ++i) {
-        enum bf_map_bpf_type type;
-        const char *str = bf_map_bpf_type_to_str(i);
-        assert_non_null(str);
-        assert_success(bf_map_bpf_type_from_str(str, &type));
-        assert_int_equal(i, type);
-    }
-
-    assert_error(bf_map_bpf_type_from_str("invalid", &type));
-}
