@@ -7,9 +7,9 @@
 
 #include <stddef.h>
 
-#include "core/front.h"
-#include "core/hook.h"
-#include "core/verdict.h"
+#include "bpfilter/front.h"
+#include "bpfilter/hook.h"
+#include "bpfilter/verdict.h"
 
 struct bf_cgen;
 struct bf_nfgroup;
@@ -23,6 +23,4 @@ struct nlmsghdr;
 struct bf_chain *bf_test_chain(enum bf_hook hook, enum bf_verdict policy);
 struct bf_cgen *bf_test_cgen(enum bf_front front, enum bf_hook hook,
                              enum bf_verdict verdict);
-struct nlmsghdr *bf_test_get_nlmsghdr(size_t nmsg, size_t *len);
-struct bf_nfgroup *bf_test_get_nfgroup(size_t nmsg, size_t *len);
 struct bf_rule *bf_test_get_rule(size_t nmatchers);
